@@ -1,6 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Intel Corporation
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -165,7 +166,7 @@ QProcessEnvironment QProcessEnvironmentPrivate::fromList(const QStringList &list
     QStringList::ConstIterator it = list.constBegin(),
                               end = list.constEnd();
     for ( ; it != end; ++it) {
-        int pos = it->indexOf(QLatin1Char('='));
+        int pos = it->indexOf(QLatin1Char('='), 1);
         if (pos < 1)
             continue;
 
@@ -249,13 +250,13 @@ QProcessEnvironment &QProcessEnvironment::operator=(const QProcessEnvironment &o
 /*!
     \fn bool QProcessEnvironment::operator !=(const QProcessEnvironment &other) const
 
-    Returns true if this and the \a other QProcessEnvironment objects are different.
+    Returns \c true if this and the \a other QProcessEnvironment objects are different.
 
     \sa operator==()
 */
 
 /*!
-    Returns true if this and the \a other QProcessEnvironment objects are equal.
+    Returns \c true if this and the \a other QProcessEnvironment objects are equal.
 
     Two QProcessEnvironment objects are considered equal if they have the same
     set of key=value pairs. The comparison of keys is done case-sensitive on
@@ -275,7 +276,7 @@ bool QProcessEnvironment::operator==(const QProcessEnvironment &other) const
 }
 
 /*!
-    Returns true if this QProcessEnvironment object is empty: that is
+    Returns \c true if this QProcessEnvironment object is empty: that is
     there are no key=value pairs set.
 
     \sa clear(), systemEnvironment(), insert()
@@ -301,7 +302,7 @@ void QProcessEnvironment::clear()
 }
 
 /*!
-    Returns true if the environment variable of name \a name is found in
+    Returns \c true if the environment variable of name \a name is found in
     this QProcessEnvironment object.
 
 
@@ -533,7 +534,11 @@ void QProcessPrivate::Channel::clear()
     MergedChannels before starting the process to activative
     this feature. You also have the option of forwarding the output of
     the running process to the calling, main process, by passing
-    ForwardedChannels as the argument.
+    ForwardedChannels as the argument. It is also possible to forward
+    only one of the output channels - typically one would use
+    ForwardedErrorChannel, but ForwardedOutputChannel also exists.
+    Note that using channel forwarding is typically a bad idea in GUI
+    applications - you should present errors graphically instead.
 
     Certain processes need special environment settings in order to
     operate. You can set environment variables for your process by
@@ -604,8 +609,8 @@ void QProcessPrivate::Channel::clear()
 /*!
     \enum QProcess::ProcessChannelMode
 
-    This enum describes the process channel modes of QProcess. Pass
-    one of these values to setProcessChannelMode() to set the
+    This enum describes the process output channel modes of QProcess.
+    Pass one of these values to setProcessChannelMode() to set the
     current read channel mode.
 
     \value SeparateChannels QProcess manages the output of the
@@ -625,6 +630,17 @@ void QProcessPrivate::Channel::clear()
     writes to its standard output and standard error will be written
     to the standard output and standard error of the main process.
 
+    \value ForwardedErrorChannel QProcess manages the standard output
+    of the running process, but forwards its standard error onto the
+    main process. This reflects the typical use of command line tools
+    as filters, where the standard output is redirected to another
+    process or a file, while standard error is printed to the console
+    for diagnostic purposes.
+    (This value was introduced in Qt 5.2.)
+
+    \value ForwardedOutputChannel Complementary to ForwardedErrorChannel.
+    (This value was introduced in Qt 5.2.)
+
     \note Windows intentionally suppresses output from GUI-only
     applications to inherited consoles.
     This does \e not apply to output redirected to files or pipes.
@@ -636,6 +652,26 @@ void QProcessPrivate::Channel::clear()
     \sa setProcessChannelMode()
 */
 
+/*!
+    \enum QProcess::InputChannelMode
+    \since 5.2
+
+    This enum describes the process input channel modes of QProcess.
+    Pass one of these values to setInputChannelMode() to set the
+    current write channel mode.
+
+    \value ManagedInputChannel QProcess manages the input of the running
+    process. This is the default input channel mode of QProcess.
+
+    \value ForwardedInputChannel QProcess forwards the input of the main
+    process onto the running process. The child process reads its standard
+    input from the same source as the main process.
+    Note that the main process must not try to read its standard input
+    while the child process is running.
+
+    \sa setInputChannelMode()
+*/
+
 /*!
     \enum QProcess::ProcessError
 
@@ -727,9 +763,11 @@ void QProcessPrivate::Channel::clear()
     \fn void QProcess::finished(int exitCode, QProcess::ExitStatus exitStatus)
 
     This signal is emitted when the process finishes. \a exitCode is the exit
-    code of the process, and \a exitStatus is the exit status.  After the
-    process has finished, the buffers in QProcess are still intact. You can
-    still read any data that the process may have written before it finished.
+    code of the process (only valid for normal exits), and \a exitStatus is
+    the exit status.
+    After the process has finished, the buffers in QProcess are still intact.
+    You can still read any data that the process may have written before it
+    finished.
 
     \sa exitStatus()
 */
@@ -762,6 +800,7 @@ QProcessPrivate::QProcessPrivate()
 {
     processChannel = QProcess::StandardOutput;
     processChannelMode = QProcess::SeparateChannels;
+    inputChannelMode = QProcess::ManagedInputChannel;
     processError = QProcess::UnknownError;
     processState = QProcess::NotRunning;
     pid = 0;
@@ -781,9 +820,6 @@ QProcessPrivate::QProcessPrivate()
     emittedBytesWritten = false;
 #ifdef Q_OS_WIN
     notifier = 0;
-    stdoutReader = 0;
-    stderrReader = 0;
-    pipeWriter = 0;
     processFinishedNotifier = 0;
 #endif // Q_OS_WIN
 #ifdef Q_OS_UNIX
@@ -816,8 +852,7 @@ void QProcessPrivate::cleanup()
         pid = 0;
     }
     if (processFinishedNotifier) {
-        processFinishedNotifier->setEnabled(false);
-        qDeleteInEventHandler(processFinishedNotifier);
+        delete processFinishedNotifier;
         processFinishedNotifier = 0;
     }
 
@@ -827,39 +862,34 @@ void QProcessPrivate::cleanup()
     dying = false;
 
     if (stdoutChannel.notifier) {
-        stdoutChannel.notifier->setEnabled(false);
-        qDeleteInEventHandler(stdoutChannel.notifier);
+        delete stdoutChannel.notifier;
         stdoutChannel.notifier = 0;
     }
     if (stderrChannel.notifier) {
-        stderrChannel.notifier->setEnabled(false);
-        qDeleteInEventHandler(stderrChannel.notifier);
+        delete stderrChannel.notifier;
         stderrChannel.notifier = 0;
     }
     if (stdinChannel.notifier) {
-        stdinChannel.notifier->setEnabled(false);
-        qDeleteInEventHandler(stdinChannel.notifier);
+        delete stdinChannel.notifier;
         stdinChannel.notifier = 0;
     }
     if (startupSocketNotifier) {
-        startupSocketNotifier->setEnabled(false);
-        qDeleteInEventHandler(startupSocketNotifier);
+        delete startupSocketNotifier;
         startupSocketNotifier = 0;
     }
     if (deathNotifier) {
-        deathNotifier->setEnabled(false);
-        qDeleteInEventHandler(deathNotifier);
+        delete deathNotifier;
         deathNotifier = 0;
     }
 #ifdef Q_OS_WIN
     if (notifier) {
-        qDeleteInEventHandler(notifier);
+        delete notifier;
         notifier = 0;
     }
 #endif
-    destroyChannel(&stdoutChannel);
-    destroyChannel(&stderrChannel);
-    destroyChannel(&stdinChannel);
+    closeChannel(&stdoutChannel);
+    closeChannel(&stderrChannel);
+    closeChannel(&stdinChannel);
     destroyPipe(childStartedPipe);
     destroyPipe(deathPipe);
 #ifdef Q_OS_UNIX
@@ -869,49 +899,63 @@ void QProcessPrivate::cleanup()
 
 /*!
     \internal
+    Returns true if we emitted readyRead().
 */
-bool QProcessPrivate::_q_canReadStandardOutput()
+bool QProcessPrivate::tryReadFromChannel(Channel *channel)
 {
     Q_Q(QProcess);
-    qint64 available = bytesAvailableFromStdout();
-    if (available == 0) {
-        if (stdoutChannel.notifier)
-            stdoutChannel.notifier->setEnabled(false);
-        destroyChannel(&stdoutChannel);
-#if defined QPROCESS_DEBUG
-        qDebug("QProcessPrivate::canReadStandardOutput(), 0 bytes available");
-#endif
+    if (channel->pipe[0] == INVALID_Q_PIPE)
+        return false;
+
+    qint64 available = bytesAvailableInChannel(channel);
+    if (available == 0)
+        available = 1;      // always try to read at least one byte
+
+    char *ptr = channel->buffer.reserve(available);
+    qint64 readBytes = readFromChannel(channel, ptr, available);
+    if (readBytes <= 0)
+        channel->buffer.chop(available);
+    if (readBytes == -2) {
+        // EWOULDBLOCK
         return false;
     }
-
-    char *ptr = outputReadBuffer.reserve(available);
-    qint64 readBytes = readFromStdout(ptr, available);
     if (readBytes == -1) {
         processError = QProcess::ReadError;
         q->setErrorString(QProcess::tr("Error reading from process"));
         emit q->error(processError);
 #if defined QPROCESS_DEBUG
-        qDebug("QProcessPrivate::canReadStandardOutput(), failed to read from the process");
+        qDebug("QProcessPrivate::tryReadFromChannel(%d), failed to read from the process", channel - &stdinChannel);
+#endif
+        return false;
+    }
+    if (readBytes == 0) {
+        // EOF
+        if (channel->notifier)
+            channel->notifier->setEnabled(false);
+        closeChannel(channel);
+#if defined QPROCESS_DEBUG
+        qDebug("QProcessPrivate::tryReadFromChannel(%d), 0 bytes available", channel - &stdinChannel);
 #endif
         return false;
     }
 #if defined QPROCESS_DEBUG
-    qDebug("QProcessPrivate::canReadStandardOutput(), read %d bytes from the process' output",
+    qDebug("QProcessPrivate::tryReadFromChannel(%d), read %d bytes from the process' output", channel - &stdinChannel
             int(readBytes));
 #endif
 
-    if (stdoutChannel.closed) {
-        outputReadBuffer.chop(readBytes);
+    if (channel->closed) {
+        channel->buffer.chop(readBytes);
         return false;
     }
 
-    outputReadBuffer.chop(available - readBytes);
+    channel->buffer.chop(available - readBytes);
 
     bool didRead = false;
+    bool isStdout = channel == &stdoutChannel;
     if (readBytes == 0) {
-        if (stdoutChannel.notifier)
-            stdoutChannel.notifier->setEnabled(false);
-    } else if (processChannel == QProcess::StandardOutput) {
+        if (channel->notifier)
+            channel->notifier->setEnabled(false);
+    } else if ((processChannel == QProcess::StandardOutput) == isStdout) {
         didRead = true;
         if (!emittedReadyRead) {
             emittedReadyRead = true;
@@ -919,53 +963,27 @@ bool QProcessPrivate::_q_canReadStandardOutput()
             emittedReadyRead = false;
         }
     }
-    emit q->readyReadStandardOutput(QProcess::QPrivateSignal());
+    if (isStdout)
+        emit q->readyReadStandardOutput(QProcess::QPrivateSignal());
+    else
+        emit q->readyReadStandardError(QProcess::QPrivateSignal());
     return didRead;
 }
 
+/*!
+    \internal
+*/
+bool QProcessPrivate::_q_canReadStandardOutput()
+{
+    return tryReadFromChannel(&stdoutChannel);
+}
+
 /*!
     \internal
 */
 bool QProcessPrivate::_q_canReadStandardError()
 {
-    Q_Q(QProcess);
-    qint64 available = bytesAvailableFromStderr();
-    if (available == 0) {
-        if (stderrChannel.notifier)
-            stderrChannel.notifier->setEnabled(false);
-        destroyChannel(&stderrChannel);
-        return false;
-    }
-
-    char *ptr = errorReadBuffer.reserve(available);
-    qint64 readBytes = readFromStderr(ptr, available);
-    if (readBytes == -1) {
-        processError = QProcess::ReadError;
-        q->setErrorString(QProcess::tr("Error reading from process"));
-        emit q->error(processError);
-        return false;
-    }
-    if (stderrChannel.closed) {
-        errorReadBuffer.chop(readBytes);
-        return false;
-    }
-
-    errorReadBuffer.chop(available - readBytes);
-
-    bool didRead = false;
-    if (readBytes == 0) {
-        if (stderrChannel.notifier)
-            stderrChannel.notifier->setEnabled(false);
-    } else if (processChannel == QProcess::StandardError) {
-        didRead = true;
-        if (!emittedReadyRead) {
-            emittedReadyRead = true;
-            emit q->readyRead();
-            emittedReadyRead = false;
-        }
-    }
-    emit q->readyReadStandardError(QProcess::QPrivateSignal());
-    return didRead;
+    return tryReadFromChannel(&stderrChannel);
 }
 
 /*!
@@ -977,17 +995,17 @@ bool QProcessPrivate::_q_canWrite()
     if (stdinChannel.notifier)
         stdinChannel.notifier->setEnabled(false);
 
-    if (writeBuffer.isEmpty()) {
+    if (stdinChannel.buffer.isEmpty()) {
 #if defined QPROCESS_DEBUG
         qDebug("QProcessPrivate::canWrite(), not writing anything (empty write buffer).");
 #endif
         return false;
     }
 
-    qint64 written = writeToStdin(writeBuffer.readPointer(),
-                                      writeBuffer.nextDataBlockSize());
+    qint64 written = writeToStdin(stdinChannel.buffer.readPointer(),
+                                      stdinChannel.buffer.nextDataBlockSize());
     if (written < 0) {
-        destroyChannel(&stdinChannel);
+        closeChannel(&stdinChannel);
         processError = QProcess::WriteError;
         q->setErrorString(QProcess::tr("Error writing to process"));
         emit q->error(processError);
@@ -999,16 +1017,16 @@ bool QProcessPrivate::_q_canWrite()
 #endif
 
     if (written != 0) {
-        writeBuffer.free(written);
+        stdinChannel.buffer.free(written);
         if (!emittedBytesWritten) {
             emittedBytesWritten = true;
             emit q->bytesWritten(written);
             emittedBytesWritten = false;
         }
     }
-    if (stdinChannel.notifier && !writeBuffer.isEmpty())
+    if (stdinChannel.notifier && !stdinChannel.buffer.isEmpty())
         stdinChannel.notifier->setEnabled(true);
-    if (writeBuffer.isEmpty() && stdinChannel.closed)
+    if (stdinChannel.buffer.isEmpty() && stdinChannel.closed)
         closeWriteChannel();
     return true;
 }
@@ -1123,19 +1141,15 @@ void QProcessPrivate::closeWriteChannel()
     qDebug("QProcessPrivate::closeWriteChannel()");
 #endif
     if (stdinChannel.notifier) {
-        extern void qDeleteInEventHandler(QObject *o);
-        stdinChannel.notifier->setEnabled(false);
-        if (stdinChannel.notifier) {
-            qDeleteInEventHandler(stdinChannel.notifier);
-            stdinChannel.notifier = 0;
-        }
+        delete stdinChannel.notifier;
+        stdinChannel.notifier = 0;
     }
 #ifdef Q_OS_WIN
     // ### Find a better fix, feeding the process little by little
     // instead.
     flushPipeWriter();
 #endif
-    destroyChannel(&stdinChannel);
+    closeChannel(&stdinChannel);
 }
 
 /*!
@@ -1226,6 +1240,34 @@ void QProcess::setProcessChannelMode(ProcessChannelMode mode)
     d->processChannelMode = mode;
 }
 
+/*!
+    \since 5.2
+
+    Returns the channel mode of the QProcess standard input channel.
+
+    \sa setInputChannelMode(), InputChannelMode
+*/
+QProcess::InputChannelMode QProcess::inputChannelMode() const
+{
+    Q_D(const QProcess);
+    return d->inputChannelMode;
+}
+
+/*!
+    \since 5.2
+
+    Sets the channel mode of the QProcess standard intput
+    channel to the \a mode specified.
+    This mode will be used the next time start() is called.
+
+    \sa inputChannelMode(), InputChannelMode
+*/
+void QProcess::setInputChannelMode(InputChannelMode mode)
+{
+    Q_D(QProcess);
+    d->inputChannelMode = mode;
+}
+
 /*!
     Returns the current read channel of the QProcess.
 
@@ -1252,10 +1294,10 @@ void QProcess::setReadChannel(ProcessChannel channel)
         QByteArray buf = d->buffer.readAll();
         if (d->processChannel == QProcess::StandardOutput) {
             for (int i = buf.size() - 1; i >= 0; --i)
-                d->outputReadBuffer.ungetChar(buf.at(i));
+                d->stdoutChannel.buffer.ungetChar(buf.at(i));
         } else {
             for (int i = buf.size() - 1; i >= 0; --i)
-                d->errorReadBuffer.ungetChar(buf.at(i));
+                d->stderrChannel.buffer.ungetChar(buf.at(i));
         }
     }
     d->processChannel = channel;
@@ -1303,7 +1345,7 @@ void QProcess::closeWriteChannel()
 {
     Q_D(QProcess);
     d->stdinChannel.closed = true; // closing
-    if (d->writeBuffer.isEmpty())
+    if (d->stdinChannel.buffer.isEmpty())
         d->closeWriteChannel();
 }
 
@@ -1315,6 +1357,10 @@ void QProcess::closeWriteChannel()
     object will be in read-only mode (calling write() will result in
     error).
 
+    To make the process read EOF right away, pass nullDevice() here.
+    This is cleaner than using closeWriteChannel() before writing any
+    data, because it can be set up prior to starting the process.
+
     If the file \a fileName does not exist at the moment start() is
     called or is not readable, starting the process will fail.
 
@@ -1338,6 +1384,10 @@ void QProcess::setStandardInputFile(const QString &fileName)
     read channel is closed: reading from it using read() will always
     fail, as will readAllStandardOutput().
 
+    To discard all standard output from the process, pass nullDevice()
+    here. This is more efficient than simply never reading the standard
+    output, as no QProcess buffers are filled.
+
     If the file \a fileName doesn't exist at the moment start() is
     called, it will be created. If it cannot be created, the starting
     will fail.
@@ -1481,16 +1531,40 @@ void QProcess::setWorkingDirectory(const QString &dir)
     d->workingDirectory = dir;
 }
 
+
 /*!
+    \deprecated
+    Use processId() instead.
+
     Returns the native process identifier for the running process, if
-    available.  If no process is currently running, 0 is returned.
+    available.  If no process is currently running, \c 0 is returned.
+
+    \note Unlike \l processId(), pid() returns an integer on Unix and a pointer on Windows.
+
+    \sa Q_PID, processId()
 */
-Q_PID QProcess::pid() const
+Q_PID QProcess::pid() const // ### Qt 6 remove or rename this method to processInformation()
 {
     Q_D(const QProcess);
     return d->pid;
 }
 
+/*!
+    \since 5.3
+
+    Returns the native process identifier for the running process, if
+    available. If no process is currently running, \c 0 is returned.
+ */
+qint64 QProcess::processId() const
+{
+    Q_D(const QProcess);
+#ifdef Q_OS_WIN
+    return d->pid ? d->pid->dwProcessId : 0;
+#else
+    return d->pid;
+#endif
+}
+
 /*! \reimp
 
     This function operates on the current read channel.
@@ -1501,8 +1575,8 @@ bool QProcess::canReadLine() const
 {
     Q_D(const QProcess);
     const QRingBuffer *readBuffer = (d->processChannel == QProcess::StandardError)
-                                    ? &d->errorReadBuffer
-                                    : &d->outputReadBuffer;
+                                    ? &d->stderrChannel.buffer
+                                    : &d->stdoutChannel.buffer;
     return readBuffer->canReadLine() || QIODevice::canReadLine();
 }
 
@@ -1523,15 +1597,15 @@ void QProcess::close()
 
 /*! \reimp
 
-   Returns true if the process is not running, and no more data is available
-   for reading; otherwise returns false.
+   Returns \c true if the process is not running, and no more data is available
+   for reading; otherwise returns \c false.
 */
 bool QProcess::atEnd() const
 {
     Q_D(const QProcess);
     const QRingBuffer *readBuffer = (d->processChannel == QProcess::StandardError)
-                                    ? &d->errorReadBuffer
-                                    : &d->outputReadBuffer;
+                                    ? &d->stderrChannel.buffer
+                                    : &d->stdoutChannel.buffer;
     return QIODevice::atEnd() && (!isOpen() || readBuffer->isEmpty());
 }
 
@@ -1548,8 +1622,8 @@ qint64 QProcess::bytesAvailable() const
 {
     Q_D(const QProcess);
     const QRingBuffer *readBuffer = (d->processChannel == QProcess::StandardError)
-                                    ? &d->errorReadBuffer
-                                    : &d->outputReadBuffer;
+                                    ? &d->stderrChannel.buffer
+                                    : &d->stdoutChannel.buffer;
 #if defined QPROCESS_DEBUG
     qDebug("QProcess::bytesAvailable() == %i (%s)", readBuffer->size(),
            (d->processChannel == QProcess::StandardError) ? "stderr" : "stdout");
@@ -1562,7 +1636,7 @@ qint64 QProcess::bytesAvailable() const
 qint64 QProcess::bytesToWrite() const
 {
     Q_D(const QProcess);
-    qint64 size = d->writeBuffer.size();
+    qint64 size = d->stdinChannel.buffer.size();
 #ifdef Q_OS_WIN
     size += d->pipeWriterBytesToWrite();
 #endif
@@ -1671,8 +1745,8 @@ QProcessEnvironment QProcess::processEnvironment() const
     Blocks until the process has started and the started() signal has
     been emitted, or until \a msecs milliseconds have passed.
 
-    Returns true if the process was started successfully; otherwise
-    returns false (if the operation timed out or if an error
+    Returns \c true if the process was started successfully; otherwise
+    returns \c false (if the operation timed out or if an error
     occurred).
 
     This function can operate without an event loop. It is
@@ -1689,10 +1763,10 @@ QProcessEnvironment QProcess::processEnvironment() const
 bool QProcess::waitForStarted(int msecs)
 {
     Q_D(QProcess);
-    if (d->processState == QProcess::Running)
-        return true;
+    if (d->processState == QProcess::Starting)
+        return d->waitForStarted(msecs);
 
-    return d->waitForStarted(msecs);
+    return d->processState == QProcess::Running;
 }
 
 /*! \reimp
@@ -1734,7 +1808,7 @@ bool QProcess::waitForBytesWritten(int msecs)
     Blocks until the process has finished and the finished() signal
     has been emitted, or until \a msecs milliseconds have passed.
 
-    Returns true if the process finished; otherwise returns false (if
+    Returns \c true if the process finished; otherwise returns \c false (if
     the operation timed out, if an error occurred, or if this QProcess
     is already finished).
 
@@ -1809,8 +1883,8 @@ qint64 QProcess::readData(char *data, qint64 maxlen)
     if (!maxlen)
         return 0;
     QRingBuffer *readBuffer = (d->processChannel == QProcess::StandardError)
-                              ? &d->errorReadBuffer
-                              : &d->outputReadBuffer;
+                              ? &d->stderrChannel.buffer
+                              : &d->stdoutChannel.buffer;
 
     if (maxlen == 1 && !readBuffer->isEmpty()) {
         int c = readBuffer->getChar();
@@ -1873,7 +1947,7 @@ qint64 QProcess::writeData(const char *data, qint64 len)
     }
 
     if (len == 1) {
-        d->writeBuffer.putChar(*data);
+        d->stdinChannel.buffer.putChar(*data);
         if (d->stdinChannel.notifier)
             d->stdinChannel.notifier->setEnabled(true);
 #if defined QPROCESS_DEBUG
@@ -1883,7 +1957,7 @@ qint64 QProcess::writeData(const char *data, qint64 len)
         return 1;
     }
 
-    char *dest = d->writeBuffer.reserve(len);
+    char *dest = d->stdinChannel.buffer.reserve(len);
     memcpy(dest, data, len);
     if (d->stdinChannel.notifier)
         d->stdinChannel.notifier->setEnabled(true);
@@ -1927,15 +2001,12 @@ QByteArray QProcess::readAllStandardError()
 }
 
 /*!
-    Starts the given \a program in a new process, if none is already
-    running, passing the command line arguments in \a arguments. The OpenMode
-    is set to \a mode.
+    Starts the given \a program in a new process, passing the command line
+    arguments in \a arguments.
 
     The QProcess object will immediately enter the Starting state. If the
     process starts successfully, QProcess will emit started(); otherwise,
-    error() will be emitted. If the QProcess object is already running a
-    process, a warning may be printed at the console, and the existing
-    process will continue running.
+    error() will be emitted.
 
     \note Processes are started asynchronously, which means the started()
     and error() signals may be delayed. Call waitForStarted() to make
@@ -1944,9 +2015,18 @@ QByteArray QProcess::readAllStandardError()
 
     \note No further splitting of the arguments is performed.
 
-    \b{Windows:} Arguments that contain spaces are wrapped in quotes.
+    \b{Windows:} The arguments are quoted and joined into a command line
+    that is compatible with the CommandLineToArgvW() Windows function.
+    For programs that have different command line quoting requirements,
+    you need to use setNativeArguments().
 
-    \sa pid(), started(), waitForStarted()
+    The OpenMode is set to \a mode.
+
+    If the QProcess object is already running a process, a warning may be
+    printed at the console, and the existing process will continue running
+    unaffected.
+
+    \sa pid(), started(), waitForStarted(), setNativeArguments()
 */
 void QProcess::start(const QString &program, const QStringList &arguments, OpenMode mode)
 {
@@ -1959,7 +2039,7 @@ void QProcess::start(const QString &program, const QStringList &arguments, OpenM
     d->program = program;
     d->arguments = arguments;
 
-    open(mode);
+    d->start(mode);
 }
 
 /*!
@@ -1969,32 +2049,31 @@ void QProcess::start(const QString &program, const QStringList &arguments, OpenM
     Starts the program set by setProgram() with arguments set by setArguments().
     The OpenMode is set to \a mode.
 
-    This method is a convenient alias to open().
-
     \sa open(), setProgram(), setArguments()
  */
 void QProcess::start(OpenMode mode)
 {
-    open(mode);
+    Q_D(QProcess);
+    if (d->processState != NotRunning) {
+        qWarning("QProcess::start: Process is already running");
+        return;
+    }
+    if (d->program.isEmpty()) {
+        qWarning("QProcess::start: program not set");
+        return;
+    }
+
+    d->start(mode);
 }
 
 /*!
-    Starts the program set by setProgram() in a new process, if none is already
-    running, passing the command line arguments set by setArguments(). The OpenMode
-    is set to \a mode.
+    Starts the program set by setProgram() with arguments set by setArguments().
+    The OpenMode is set to \a mode.
 
-    The QProcess object will immediately enter the Starting state. If the
-    process starts successfully, QProcess will emit started(); otherwise,
-    error() will be emitted. If the QProcess object is already running a
-    process, a warning may be printed at the console, the function will return false,
-    and the existing process will continue running.
-
-    \note Processes are started asynchronously, which means the started()
-    and error() signals may be delayed. Call waitForStarted() to make
-    sure the process has started (or has failed to start) and those signals
-    have been emitted. In this regard, a true return value merly means the process
-    was correcty initialized, not that the program was actually started.
+    This method is an alias for start(), and exists only to fully implement
+    the interface defined by QIODevice.
 
+    \sa start(), setProgram(), setArguments()
 */
 bool QProcess::open(OpenMode mode)
 {
@@ -2008,34 +2087,39 @@ bool QProcess::open(OpenMode mode)
         return false;
     }
 
+    d->start(mode);
+    return true;
+}
+
+void QProcessPrivate::start(QIODevice::OpenMode mode)
+{
+    Q_Q(QProcess);
 #if defined QPROCESS_DEBUG
     qDebug() << "QProcess::start(" << program << ',' << arguments << ',' << mode << ')';
 #endif
 
-    d->outputReadBuffer.clear();
-    d->errorReadBuffer.clear();
+    stdoutChannel.buffer.clear();
+    stderrChannel.buffer.clear();
 
-    if (d->stdinChannel.type != QProcessPrivate::Channel::Normal)
-        mode &= ~WriteOnly;     // not open for writing
-    if (d->stdoutChannel.type != QProcessPrivate::Channel::Normal &&
-        (d->stderrChannel.type != QProcessPrivate::Channel::Normal ||
-         d->processChannelMode == MergedChannels))
-        mode &= ~ReadOnly;      // not open for reading
+    if (stdinChannel.type != QProcessPrivate::Channel::Normal)
+        mode &= ~QIODevice::WriteOnly;     // not open for writing
+    if (stdoutChannel.type != QProcessPrivate::Channel::Normal &&
+        (stderrChannel.type != QProcessPrivate::Channel::Normal ||
+         processChannelMode == QProcess::MergedChannels))
+        mode &= ~QIODevice::ReadOnly;      // not open for reading
     if (mode == 0)
-        mode = Unbuffered;
-    QIODevice::open(mode);
+        mode = QIODevice::Unbuffered;
+    q->QIODevice::open(mode);
 
-    d->stdinChannel.closed = false;
-    d->stdoutChannel.closed = false;
-    d->stderrChannel.closed = false;
+    stdinChannel.closed = false;
+    stdoutChannel.closed = false;
+    stderrChannel.closed = false;
 
-    d->exitCode = 0;
-    d->exitStatus = NormalExit;
-    d->processError = QProcess::UnknownError;
-    d->errorString.clear();
-    d->startProcess();
-
-    return true;
+    exitCode = 0;
+    exitStatus = QProcess::NormalExit;
+    processError = QProcess::UnknownError;
+    errorString.clear();
+    startProcess();
 }
 
 
@@ -2082,29 +2166,28 @@ static QStringList parseCombinedArgString(const QString &program)
 /*!
     \overload
 
-    Starts the command \a command in a new process, if one is not already
-    running. \a command is a single string of text containing both the
-    program name and its arguments. The arguments are separated by one or
-    more spaces. For example:
+    Starts the command \a command in a new process.
+    The OpenMode is set to \a mode.
+
+    \a command is a single string of text containing both the program name
+    and its arguments. The arguments are separated by one or more spaces.
+    For example:
 
     \snippet code/src_corelib_io_qprocess.cpp 5
 
-    The \a command string can also contain quotes, to ensure that arguments
-    containing spaces are correctly supplied to the new process. For example:
+    Arguments containing spaces must be quoted to be correctly supplied to
+    the new process. For example:
 
     \snippet code/src_corelib_io_qprocess.cpp 6
 
-    If the QProcess object is already running a process, a warning may be
-    printed at the console, and the existing process will continue running.
-
-    Note that, on Windows, quotes need to be both escaped and quoted.
-    For example, the above code would be specified in the following
-    way to ensure that \c{"My Documents"} is used as the argument to
-    the \c dir executable:
+    Literal quotes in the \a command string are represented by triple quotes.
+    For example:
 
     \snippet code/src_corelib_io_qprocess.cpp 7
 
-    The OpenMode is set to \a mode.
+    After the \a command string has been split and unquoted, this function
+    behaves like the overload which takes the arguments as a string list.
+
 */
 void QProcess::start(const QString &command, OpenMode mode)
 {
@@ -2124,6 +2207,8 @@ void QProcess::start(const QString &command, OpenMode mode)
 }
 
 /*!
+    \since 5.0
+
     Returns the program the process was last started with.
 
     \sa start()
@@ -2138,7 +2223,7 @@ QString QProcess::program() const
     \since 5.1
 
     Set the \a program to use when starting the process.
-    That function must be call before open()
+    This function must be called before start().
 
     \sa start(), setArguments(), program()
 */
@@ -2153,6 +2238,8 @@ void QProcess::setProgram(const QString &program)
 }
 
 /*!
+    \since 5.0
+
     Returns the command line arguments the process was last started with.
 
     \sa start()
@@ -2167,7 +2254,7 @@ QStringList QProcess::arguments() const
     \since 5.1
 
     Set the \a arguments to pass to the called program when starting the process.
-    That function must be call before  open()
+    This function must be called before start().
 
     \sa start(), setProgram(), arguments()
 */
@@ -2219,6 +2306,8 @@ void QProcess::kill()
 
 /*!
     Returns the exit code of the last process that finished.
+
+    This value is not valid unless exitStatus() returns NormalExit.
 */
 int QProcess::exitCode() const
 {
@@ -2250,11 +2339,13 @@ QProcess::ExitStatus QProcess::exitStatus() const
     The environment and working directory are inherited from the calling
     process.
 
-    On Windows, arguments that contain spaces are wrapped in quotes.
+    Argument handling is identical to the respective start() overload.
 
     If the process cannot be started, -2 is returned. If the process
     crashes, -1 is returned. Otherwise, the process' exit code is
     returned.
+
+    \sa start()
 */
 int QProcess::execute(const QString &program, const QStringList &arguments)
 {
@@ -2269,15 +2360,21 @@ int QProcess::execute(const QString &program, const QStringList &arguments)
 /*!
     \overload
 
-    Starts the program \a program in a new process. \a program is a
-    single string of text containing both the program name and its
-    arguments. The arguments are separated by one or more spaces.
+    Starts the program \a command in a new process, waits for it to finish,
+    and then returns the exit code.
+
+    Argument handling is identical to the respective start() overload.
+
+    After the \a command string has been split and unquoted, this function
+    behaves like the overload which takes the arguments as a string list.
+
+    \sa start()
 */
-int QProcess::execute(const QString &program)
+int QProcess::execute(const QString &command)
 {
     QProcess process;
     process.setReadChannelMode(ForwardedChannels);
-    process.start(program);
+    process.start(command);
     if (!process.waitForFinished(-1))
         return -2;
     return process.exitStatus() == QProcess::NormalExit ? process.exitCode() : -1;
@@ -2285,55 +2382,43 @@ int QProcess::execute(const QString &program)
 
 /*!
     Starts the program \a program with the arguments \a arguments in a
-    new process, and detaches from it. Returns true on success;
-    otherwise returns false. If the calling process exits, the
-    detached process will continue to live.
+    new process, and detaches from it. Returns \c true on success;
+    otherwise returns \c false. If the calling process exits, the
+    detached process will continue to run unaffected.
 
-    Note that arguments that contain spaces are not passed to the
-    process as separate arguments.
+    Argument handling is identical to the respective start() overload.
 
     \b{Unix:} The started process will run in its own session and act
     like a daemon.
 
-    \b{Windows:} Arguments that contain spaces are wrapped in quotes.
-    The started process will run as a regular standalone process.
-
     The process will be started in the directory \a workingDirectory.
+    If \a workingDirectory is empty, the working directory is inherited
+    from the calling process.
 
     \note On QNX, this may cause all application threads to
     temporarily freeze.
 
     If the function is successful then *\a pid is set to the process
     identifier of the started process.
+
+    \sa start()
 */
 bool QProcess::startDetached(const QString &program,
-			     const QStringList &arguments,
-			     const QString &workingDirectory,
+                             const QStringList &arguments,
+                             const QString &workingDirectory,
                              qint64 *pid)
 {
     return QProcessPrivate::startDetached(program,
-					  arguments,
-					  workingDirectory,
-					  pid);
+                                          arguments,
+                                          workingDirectory,
+                                          pid);
 }
 
 /*!
-    Starts the program \a program with the given \a arguments in a
-    new process, and detaches from it. Returns true on success;
-    otherwise returns false. If the calling process exits, the
-    detached process will continue to live.
-
-    \note Arguments that contain spaces are not passed to the
-    process as separate arguments.
-
-    \b{Unix:} The started process will run in its own session and act
-    like a daemon.
-
-    \b{Windows:} Arguments that contain spaces are wrapped in quotes.
-    The started process will run as a regular standalone process.
+    \internal
 */
 bool QProcess::startDetached(const QString &program,
-			     const QStringList &arguments)
+                             const QStringList &arguments)
 {
     return QProcessPrivate::startDetached(program, arguments);
 }
@@ -2341,16 +2426,19 @@ bool QProcess::startDetached(const QString &program,
 /*!
     \overload
 
-    Starts the program \a program in a new process. \a program is a
-    single string of text containing both the program name and its
-    arguments. The arguments are separated by one or more spaces.
+    Starts the command \a command in a new process, and detaches from it.
+    Returns \c true on success; otherwise returns \c false.
 
-    The \a program string can also contain quotes, to ensure that arguments
-    containing spaces are correctly supplied to the new process.
+    Argument handling is identical to the respective start() overload.
+
+    After the \a command string has been split and unquoted, this function
+    behaves like the overload which takes the arguments as a string list.
+
+    \sa start()
 */
-bool QProcess::startDetached(const QString &program)
+bool QProcess::startDetached(const QString &command)
 {
-    QStringList args = parseCombinedArgString(program);
+    QStringList args = parseCombinedArgString(command);
     if (args.isEmpty())
         return false;
 
@@ -2420,6 +2508,25 @@ QStringList QProcess::systemEnvironment()
     \sa QProcess::systemEnvironment()
 */
 
+/*!
+    \since 5.2
+
+    \brief The null device of the operating system.
+
+    The returned file path uses native directory separators.
+
+    \sa QProcess::setStandardInputFile(), QProcess::setStandardOutputFile(),
+        QProcess::setStandardErrorFile()
+*/
+QString QProcess::nullDevice()
+{
+#ifdef Q_OS_WIN
+    return QStringLiteral("\\\\.\\NUL");
+#else
+    return QStringLiteral("/dev/null");
+#endif
+}
+
 /*!
     \typedef Q_PID
     \relates QProcess
