@@ -391,6 +391,10 @@ static void qDBusNewConnection(DBusServer *server, DBusConnection *connection, v
     q_dbus_connection_ref(connection);
     QDBusConnectionPrivate *serverConnection = static_cast<QDBusConnectionPrivate *>(data);
 
+    // allow anonymous authentication
+    if (serverConnection->anonymousAuthenticationAllowed)
+        q_dbus_connection_set_allow_anonymous(connection, true);
+
     QDBusConnectionPrivate *newConnection = new QDBusConnectionPrivate(serverConnection->parent());
     QMutexLocker locker(&QDBusConnectionManager::instance()->mutex);
     QDBusConnectionManager::instance()->setConnection(QLatin1String("QDBusServer-") + QString::number(reinterpret_cast<qulonglong>(newConnection)), newConnection);
@@ -587,46 +591,22 @@ bool QDBusConnectionPrivate::handleMessage(const QDBusMessage &amsg)
     return false;
 }
 
-static void garbageCollectChildren(QDBusConnectionPrivate::ObjectTreeNode &node)
-{
-    int size = node.children.count();
-    if (node.activeChildren == 0) {
-        // easy case
-        node.children.clear();
-    } else if (size > node.activeChildren * 3 || (size > 20 && size * 2 > node.activeChildren * 3)) {
-        // rewrite the vector, keeping only the active children
-        // if the vector is large (> 20 items) and has one third of inactives
-        // or if the vector is small and has two thirds of inactives.
-        QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator end = node.children.end();
-        QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator it = node.children.begin();
-        QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator tgt = it;
-        for ( ; it != end; ++it) {
-            if (it->isActive())
-                *tgt++ = qMove(*it);
-        }
-        ++tgt;
-        node.children.erase(tgt, end);
-    }
-}
-
 static void huntAndDestroy(QObject *needle, QDBusConnectionPrivate::ObjectTreeNode &haystack)
 {
     QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator it = haystack.children.begin();
-    QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator end = haystack.children.end();
-    for ( ; it != end; ++it) {
-        if (!it->isActive())
-            continue;
+
+    while (it != haystack.children.end()) {
         huntAndDestroy(needle, *it);
         if (!it->isActive())
-            --haystack.activeChildren;
+            it = haystack.children.erase(it);
+        else
+            it++;
     }
 
     if (needle == haystack.obj) {
         haystack.obj = 0;
         haystack.flags = 0;
     }
-
-    garbageCollectChildren(haystack);
 }
 
 static void huntAndUnregister(const QStringList &pathComponents, int i, QDBusConnection::UnregisterMode mode,
@@ -639,7 +619,6 @@ static void huntAndUnregister(const QStringList &pathComponents, int i, QDBusCon
 
         if (mode == QDBusConnection::UnregisterTree) {
             // clear the sub-tree as well
-            node->activeChildren = 0;
             node->children.clear();  // can't disconnect the objects because we really don't know if they can
                             // be found somewhere else in the path too
         }
@@ -648,14 +627,12 @@ static void huntAndUnregister(const QStringList &pathComponents, int i, QDBusCon
         QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator end = node->children.end();
         QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator it =
             std::lower_bound(node->children.begin(), end, pathComponents.at(i));
-        if (it == end || it->name != pathComponents.at(i) || !it->isActive())
+        if (it == end || it->name != pathComponents.at(i))
             return;              // node not found
 
         huntAndUnregister(pathComponents, i + 1, mode, it);
         if (!it->isActive())
-            --node->activeChildren;
-
-        garbageCollectChildren(*node);
+            node->children.erase(it);
     }
 }
 
@@ -721,7 +698,8 @@ static int findSlot(const QMetaObject *mo, const QByteArray &name, int flags,
         if (isAsync && returnType != QMetaType::Void)
             continue;
 
-        int inputCount = qDBusParametersForMethod(mm, metaTypes);
+        QString errorMsg;
+        int inputCount = qDBusParametersForMethod(mm, metaTypes, errorMsg);
         if (inputCount == -1)
             continue;           // problem parsing
 
@@ -1012,9 +990,9 @@ void QDBusConnectionPrivate::deliverCall(QObject *object, int /*flags*/, const Q
         fail = object->qt_metacall(QMetaObject::InvokeMetaMethod,
                                    slotIdx, params.data()) >= 0;
         QDBusConnectionPrivate::setSender(0);
-	// the object might be deleted in the slot
-	if (!ptr.isNull())
-	    QDBusContextPrivate::set(object, old);
+        // the object might be deleted in the slot
+        if (!ptr.isNull())
+            QDBusContextPrivate::set(object, old);
     }
 
     // do we create a reply? Only if the caller is waiting for a reply and one hasn't been sent
@@ -1040,7 +1018,8 @@ extern bool qDBusInitThreads();
 QDBusConnectionPrivate::QDBusConnectionPrivate(QObject *p)
     : QObject(p), ref(1), capabilities(0), mode(InvalidMode), connection(0), server(0), busService(0),
       watchAndTimeoutLock(QMutex::Recursive),
-      rootNode(QString(QLatin1Char('/')))
+      rootNode(QString(QLatin1Char('/'))),
+      anonymousAuthenticationAllowed(false)
 {
     static const bool threads = q_dbus_threads_init_default();
     static const int debugging = qgetenv("QDBUS_DEBUG").toInt();
@@ -1118,6 +1097,9 @@ void QDBusConnectionPrivate::closeConnection()
                 ;
         }
     }
+
+    qDeleteAll(pendingCalls);
+
     qDBusDebug() << this << "Disconnected";
 }
 
@@ -1312,7 +1294,8 @@ int QDBusConnectionPrivate::findSlot(QObject* obj, const QByteArray &normalizedN
     if (midx == -1)
         return -1;
 
-    int inputCount = qDBusParametersForMethod(obj->metaObject()->method(midx), params);
+    QString errorMsg;
+    int inputCount = qDBusParametersForMethod(obj->metaObject()->method(midx), params, errorMsg);
     if ( inputCount == -1 || inputCount + 1 != params.count() )
         return -1;              // failed to parse or invalid arguments or output arguments
 
@@ -1681,13 +1664,13 @@ static dbus_int32_t server_slot = -1;
 
 void QDBusConnectionPrivate::setServer(DBusServer *s, const QDBusErrorInternal &error)
 {
+    mode = ServerMode;
     if (!s) {
         handleError(error);
         return;
     }
 
     server = s;
-    mode = ServerMode;
 
     dbus_bool_t data_allocated = q_dbus_server_allocate_data_slot(&server_slot);
     if (data_allocated && server_slot < 0)
@@ -1718,13 +1701,13 @@ void QDBusConnectionPrivate::setServer(DBusServer *s, const QDBusErrorInternal &
 
 void QDBusConnectionPrivate::setPeer(DBusConnection *c, const QDBusErrorInternal &error)
 {
+    mode = PeerMode;
     if (!c) {
         handleError(error);
         return;
     }
 
     connection = c;
-    mode = PeerMode;
 
     q_dbus_connection_set_exit_on_disconnect(connection, false);
     q_dbus_connection_set_watch_functions(connection,
@@ -1771,13 +1754,13 @@ static QDBusConnection::ConnectionCapabilities connectionCapabilies(DBusConnecti
 
 void QDBusConnectionPrivate::setConnection(DBusConnection *dbc, const QDBusErrorInternal &error)
 {
+    mode = ClientMode;
     if (!dbc) {
         handleError(error);
         return;
     }
 
     connection = dbc;
-    mode = ClientMode;
 
     const char *service = q_dbus_bus_get_unique_name(connection);
     Q_ASSERT(service);
@@ -1828,7 +1811,6 @@ static void qDBusResultReceived(DBusPendingCall *pending, void *user_data)
 void QDBusConnectionPrivate::waitForFinished(QDBusPendingCallPrivate *pcall)
 {
     Q_ASSERT(pcall->pending);
-    Q_ASSERT(!pcall->autoDelete);
     //Q_ASSERT(pcall->mutex.isLocked()); // there's no such function
 
     if (pcall->waitingForFinished) {
@@ -1844,23 +1826,24 @@ void QDBusConnectionPrivate::waitForFinished(QDBusPendingCallPrivate *pcall)
             // QDBusConnectionPrivate::processFinishedCall() is called automatically
         }
         pcall->mutex.lock();
+
+        if (pcall->pending) {
+            q_dbus_pending_call_unref(pcall->pending);
+            pcall->pending = 0;
+        }
+
         pcall->waitForFinishedCondition.wakeAll();
     }
 }
 
-// this function is called only in a Q_ASSERT
-static inline Q_DECL_UNUSED bool waitingForFinishedIsSet(QDBusPendingCallPrivate *call)
-{
-    const QMutexLocker locker(&call->mutex);
-    return call->waitingForFinished;
-}
-
 void QDBusConnectionPrivate::processFinishedCall(QDBusPendingCallPrivate *call)
 {
     QDBusConnectionPrivate *connection = const_cast<QDBusConnectionPrivate *>(call->connection);
 
     QMutexLocker locker(&call->mutex);
 
+    connection->pendingCalls.removeOne(call);
+
     QDBusMessage &msg = call->replyMessage;
     if (call->pending) {
         // decode the message
@@ -1890,9 +1873,10 @@ void QDBusConnectionPrivate::processFinishedCall(QDBusPendingCallPrivate *call)
             qDBusDebug() << "Deliver failed!";
     }
 
-    if (call->pending)
+    if (call->pending && !call->waitingForFinished) {
         q_dbus_pending_call_unref(call->pending);
-    call->pending = 0;
+        call->pending = 0;
+    }
 
     locker.unlock();
 
@@ -1903,10 +1887,8 @@ void QDBusConnectionPrivate::processFinishedCall(QDBusPendingCallPrivate *call)
     if (msg.type() == QDBusMessage::ErrorMessage)
         emit connection->callWithCallbackFailed(QDBusError(msg), call->sentMessage);
 
-    if (call->autoDelete) {
-        Q_ASSERT(!waitingForFinishedIsSet(call)); // can't wait on a call with autoDelete!
+    if (!call->ref.deref())
         delete call;
-    }
 }
 
 int QDBusConnectionPrivate::send(const QDBusMessage& message)
@@ -1989,7 +1971,7 @@ QDBusMessage QDBusConnectionPrivate::sendWithReply(const QDBusMessage &message,
 
         return amsg;
     } else { // use the event loop
-        QDBusPendingCallPrivate *pcall = sendWithReplyAsync(message, timeout);
+        QDBusPendingCallPrivate *pcall = sendWithReplyAsync(message, 0, 0, 0, timeout);
         Q_ASSERT(pcall);
 
         if (pcall->replyMessage.type() == QDBusMessage::InvalidMessage) {
@@ -2005,6 +1987,10 @@ QDBusMessage QDBusConnectionPrivate::sendWithReply(const QDBusMessage &message,
         QDBusMessage reply = pcall->replyMessage;
         lastError = QDBusError(reply);      // set or clear error
 
+        bool r = pcall->ref.deref();
+        Q_ASSERT(!r);
+        Q_UNUSED(r);
+
         delete pcall;
         return reply;
     }
@@ -2044,19 +2030,55 @@ QDBusMessage QDBusConnectionPrivate::sendWithReplyLocal(const QDBusMessage &mess
 }
 
 QDBusPendingCallPrivate *QDBusConnectionPrivate::sendWithReplyAsync(const QDBusMessage &message,
-                                                                    int timeout)
+                                                                    QObject *receiver, const char *returnMethod,
+                                                                    const char *errorMethod, int timeout)
 {
     if (isServiceRegisteredByThread(message.service())) {
         // special case for local calls
         QDBusPendingCallPrivate *pcall = new QDBusPendingCallPrivate(message, this);
         pcall->replyMessage = sendWithReplyLocal(message);
+        if (receiver && returnMethod)
+            pcall->setReplyCallback(receiver, returnMethod);
 
+        if (errorMethod) {
+            pcall->watcherHelper = new QDBusPendingCallWatcherHelper;
+            connect(pcall->watcherHelper, SIGNAL(error(QDBusError,QDBusMessage)), receiver, errorMethod,
+                    Qt::QueuedConnection);
+            pcall->watcherHelper->moveToThread(thread());
+        }
+
+        if ((receiver && returnMethod) || errorMethod) {
+           // no one waiting, will delete pcall in processFinishedCall()
+           pcall->ref.store(1);
+        } else {
+           // set double ref to prevent race between processFinishedCall() and ref counting
+           // by QDBusPendingCall::QExplicitlySharedDataPointer<QDBusPendingCallPrivate>
+           pcall->ref.store(2);
+        }
+        processFinishedCall(pcall);
         return pcall;
     }
 
     checkThread();
     QDBusPendingCallPrivate *pcall = new QDBusPendingCallPrivate(message, this);
-    pcall->ref.store(0);
+    if (receiver && returnMethod)
+        pcall->setReplyCallback(receiver, returnMethod);
+
+    if (errorMethod) {
+        pcall->watcherHelper = new QDBusPendingCallWatcherHelper;
+        connect(pcall->watcherHelper, SIGNAL(error(QDBusError,QDBusMessage)), receiver, errorMethod,
+                Qt::QueuedConnection);
+        pcall->watcherHelper->moveToThread(thread());
+    }
+
+    if ((receiver && returnMethod) || errorMethod) {
+       // no one waiting, will delete pcall in processFinishedCall()
+       pcall->ref.store(1);
+    } else {
+       // set double ref to prevent race between processFinishedCall() and ref counting
+       // by QDBusPendingCall::QExplicitlySharedDataPointer<QDBusPendingCallPrivate>
+       pcall->ref.store(2);
+    }
 
     QDBusError error;
     DBusMessage *msg = QDBusMessagePrivate::toDBusMessage(message, capabilities, &error);
@@ -2067,6 +2089,7 @@ QDBusPendingCallPrivate *QDBusConnectionPrivate::sendWithReplyAsync(const QDBusM
                  qPrintable(error.message()));
         pcall->replyMessage = QDBusMessage::createError(error);
         lastError = error;
+        processFinishedCall(pcall);
         return pcall;
     }
 
@@ -2081,6 +2104,10 @@ QDBusPendingCallPrivate *QDBusConnectionPrivate::sendWithReplyAsync(const QDBusM
             pcall->pending = pending;
             q_dbus_pending_call_set_notify(pending, qDBusResultReceived, pcall, 0);
 
+            // DBus won't notify us when a peer disconnects so we need to track these ourselves
+            if (mode == QDBusConnectionPrivate::PeerMode)
+                pendingCalls.append(pcall);
+
             return pcall;
         } else {
             // we're probably disconnected at this point
@@ -2092,46 +2119,10 @@ QDBusPendingCallPrivate *QDBusConnectionPrivate::sendWithReplyAsync(const QDBusM
 
     q_dbus_message_unref(msg);
     pcall->replyMessage = QDBusMessage::createError(error);
+    processFinishedCall(pcall);
     return pcall;
 }
 
-int QDBusConnectionPrivate::sendWithReplyAsync(const QDBusMessage &message, QObject *receiver,
-                                               const char *returnMethod, const char *errorMethod,
-                                               int timeout)
-{
-    QDBusPendingCallPrivate *pcall = sendWithReplyAsync(message, timeout);
-    Q_ASSERT(pcall);
-
-    // has it already finished with success (dispatched locally)?
-    if (pcall->replyMessage.type() == QDBusMessage::ReplyMessage) {
-        pcall->setReplyCallback(receiver, returnMethod);
-        processFinishedCall(pcall);
-        delete pcall;
-        return 1;
-    }
-
-    // either it hasn't finished or it has finished with error
-    if (errorMethod) {
-        pcall->watcherHelper = new QDBusPendingCallWatcherHelper;
-        connect(pcall->watcherHelper, SIGNAL(error(QDBusError,QDBusMessage)), receiver, errorMethod,
-                Qt::QueuedConnection);
-        pcall->watcherHelper->moveToThread(thread());
-    }
-
-    // has it already finished and is an error reply message?
-    if (pcall->replyMessage.type() == QDBusMessage::ErrorMessage) {
-        processFinishedCall(pcall);
-        delete pcall;
-        return 1;
-    }
-
-    pcall->autoDelete = true;
-    pcall->ref.ref();
-    pcall->setReplyCallback(receiver, returnMethod);
-
-    return 1;
-}
-
 bool QDBusConnectionPrivate::connectSignal(const QString &service,
                                            const QString &path, const QString &interface, const QString &name,
                                            const QStringList &argumentMatch, const QString &signature,
@@ -2497,12 +2488,15 @@ void QDBusConnectionPrivate::unregisterServiceNoLock(const QString &serviceName)
     serviceNames.removeAll(serviceName);
 }
 
-bool QDBusConnectionPrivate::isServiceRegisteredByThread(const QString &serviceName) const
+bool QDBusConnectionPrivate::isServiceRegisteredByThread(const QString &serviceName)
 {
     if (!serviceName.isEmpty() && serviceName == baseService)
         return true;
-    QStringList copy = serviceNames;
-    return copy.contains(serviceName);
+    if (serviceName == dbusServiceString())
+        return false;
+
+    QDBusReadLocker locker(UnregisterServiceAction, this);
+    return serviceNames.contains(serviceName);
 }
 
 void QDBusConnectionPrivate::postEventToThread(int action, QObject *object, QEvent *ev)
