@@ -52,12 +52,19 @@
 #include <QtGui/private/qopengl_p.h>
 #include <QtGui/private/qwindow_p.h>
 #include <QtGui/QScreen>
+#include <qpa/qplatformnativeinterface.h>
 
 #include <private/qopenglextensions_p.h>
 #include <private/qopenglversionfunctionsfactory_p.h>
 
+#include <private/qopengltexturehelper_p.h>
+
 #include <QDebug>
 
+#ifndef QT_OPENGL_ES_2
+#include <QOpenGLFunctions_1_0>
+#endif
+
 QT_BEGIN_NAMESPACE
 
 class QOpenGLVersionProfilePrivate
@@ -160,9 +167,9 @@ void QOpenGLVersionProfile::setVersion(int majorVersion, int minorVersion)
 }
 
 /*!
-    Returns the OpenGL profile. Only make sense if profiles are supported by this version.
+    Returns the OpenGL profile. Only makes sense if profiles are supported by this version.
 
-    \sa setProfile(), supportsProfiles()
+    \sa setProfile()
 */
 QSurfaceFormat::OpenGLContextProfile QOpenGLVersionProfile::profile() const
 {
@@ -170,9 +177,10 @@ QSurfaceFormat::OpenGLContextProfile QOpenGLVersionProfile::profile() const
 }
 
 /*!
-    Sets the profile.  Only make sense if profiles are supported by this version.
+    Sets the OpenGL profile \a profile. Only makes sense if profiles are supported by
+    this version.
 
-    \sa profile(), supportsProfiles()
+    \sa profile()
 */
 void QOpenGLVersionProfile::setProfile(QSurfaceFormat::OpenGLContextProfile profile)
 {
@@ -180,7 +188,7 @@ void QOpenGLVersionProfile::setProfile(QSurfaceFormat::OpenGLContextProfile prof
 }
 
 /*!
-    Returns true if profiles are supported by the OpenGL version returned by version(). Only
+    Returns \c true if profiles are supported by the OpenGL version returned by version(). Only
     OpenGL versions >= 3.2 support profiles.
 
     \sa profile(), version()
@@ -192,7 +200,7 @@ bool QOpenGLVersionProfile::hasProfiles() const
 }
 
 /*!
-    Returns true is the OpenGL version returned by version() contains deprecated functions
+    Returns \c true is the OpenGL version returned by version() contains deprecated functions
     and does not support profiles i.e. if the OpenGL version is <= 3.1.
 */
 bool QOpenGLVersionProfile::isLegacyVersion() const
@@ -201,8 +209,8 @@ bool QOpenGLVersionProfile::isLegacyVersion() const
 }
 
 /*!
-    Returns true if the version number is valid. Note that for a default constructed
-    QOpenGLVersionProfile object this function will return false.
+    Returns \c true if the version number is valid. Note that for a default constructed
+    QOpenGLVersionProfile object this function will return \c false.
 
     \sa setVersion(), version()
 */
@@ -226,6 +234,7 @@ public:
 };
 
 static QThreadStorage<QGuiGLThreadContext *> qwindow_context_storage;
+static QOpenGLContext *global_share_context = 0;
 
 #ifndef QT_NO_DEBUG
 QHash<QOpenGLContext *, bool> QOpenGLContextPrivate::makeCurrentTracker;
@@ -243,8 +252,8 @@ QMutex QOpenGLContextPrivate::makeCurrentTrackerMutex;
     To set up a context, set its screen and format such that they match those
     of the surface or surfaces with which the context is meant to be used, if
     necessary make it share resources with other contexts with
-    setShareContext(), and finally call create(). Use isValid() to check if the
-    context was successfully initialized.
+    setShareContext(), and finally call create(). Use the return value or isValid()
+    to check if the context was successfully initialized.
 
     A context can be made current against a given surface by calling
     makeCurrent(). When OpenGL rendering is done, call swapBuffers() to swap
@@ -254,12 +263,10 @@ QMutex QOpenGLContextPrivate::makeCurrentTrackerMutex;
     rendering a new frame, after calling swapBuffers().
 
     If the context is temporarily not needed, such as when the application is
-    not rendering, it can be useful to call destroy() to free resources.
-    However, if you do so you will need to call create() again before the
-    context can be used, and you might need to recreate any OpenGL resources
-    and reinitialize the OpenGL state. You can connect to the
-    aboutToBeDestroyed() signal to clean up any resources that have been
-    allocated with different ownership from the QOpenGLContext itself.
+    not rendering, it can be useful to delete it in order to free resources.
+    You can connect to the aboutToBeDestroyed() signal to clean up any
+    resources that have been allocated with different ownership from the
+    QOpenGLContext itself.
 
     Once a QOpenGLContext has been made current, you can render to it in a
     platform independent way by using Qt's OpenGL enablers such as
@@ -270,7 +277,7 @@ QMutex QOpenGLContextPrivate::makeCurrentTrackerMutex;
     or OpenGL ES 1.x.
 
     For more information about the OpenGL API, refer to the official
-    \l{OpenGL documentation}.
+    \l{http://www.opengl.org}{OpenGL documentation}.
 
     For an example of how to use QOpenGLContext see the
     \l{OpenGL Window Example}{OpenGL Window} example.
@@ -328,38 +335,62 @@ QOpenGLContext *QOpenGLContextPrivate::setCurrentContext(QOpenGLContext *context
     return previous;
 }
 
+/*!
+    \internal
+
+    This function is used by the Qt WebEngine to set up context sharing
+    across multiple windows. Do not use it for any other purpose.
+*/
+void QOpenGLContextPrivate::setGlobalShareContext(QOpenGLContext *context)
+{
+    global_share_context = context;
+}
+
+/*!
+    \internal
+*/
+QOpenGLContext *QOpenGLContextPrivate::globalShareContext()
+{
+    return global_share_context;
+}
+
 int QOpenGLContextPrivate::maxTextureSize()
 {
     if (max_texture_size != -1)
         return max_texture_size;
 
-    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
+    Q_Q(QOpenGLContext);
+    QOpenGLFunctions *funcs = q->functions();
+    funcs->glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
 
-#if defined(QT_OPENGL_ES)
-    return max_texture_size;
-#else
-    GLenum proxy = GL_PROXY_TEXTURE_2D;
+#ifndef QT_OPENGL_ES
+    if (!q->isOpenGLES()) {
+        GLenum proxy = GL_PROXY_TEXTURE_2D;
 
-    GLint size;
-    GLint next = 64;
-    glTexImage2D(proxy, 0, GL_RGBA, next, next, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
-    glGetTexLevelParameteriv(proxy, 0, GL_TEXTURE_WIDTH, &size);
-    if (size == 0) {
-        return max_texture_size;
+        GLint size;
+        GLint next = 64;
+        funcs->glTexImage2D(proxy, 0, GL_RGBA, next, next, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
+        QOpenGLFunctions_1_0 *gl1funcs = q->versionFunctions<QOpenGLFunctions_1_0>();
+        gl1funcs->initializeOpenGLFunctions();
+        gl1funcs->glGetTexLevelParameteriv(proxy, 0, GL_TEXTURE_WIDTH, &size);
+        if (size == 0) {
+            return max_texture_size;
+        }
+        do {
+            size = next;
+            next = size * 2;
+
+            if (next > max_texture_size)
+                break;
+            funcs->glTexImage2D(proxy, 0, GL_RGBA, next, next, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
+            gl1funcs->glGetTexLevelParameteriv(proxy, 0, GL_TEXTURE_WIDTH, &next);
+        } while (next > size);
+
+        max_texture_size = size;
     }
-    do {
-        size = next;
-        next = size * 2;
+#endif // QT_OPENGL_ES
 
-        if (next > max_texture_size)
-            break;
-        glTexImage2D(proxy, 0, GL_RGBA, next, next, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
-        glGetTexLevelParameteriv(proxy, 0, GL_TEXTURE_WIDTH, &next);
-    } while (next > size);
-
-    max_texture_size = size;
     return max_texture_size;
-#endif
 }
 
 /*!
@@ -376,7 +407,7 @@ QOpenGLContext* QOpenGLContext::currentContext()
 }
 
 /*!
-    Returns true if the \a first and \a second contexts are sharing OpenGL resources.
+    Returns \c true if the \a first and \a second contexts are sharing OpenGL resources.
 */
 bool QOpenGLContext::areSharing(QOpenGLContext *first, QOpenGLContext *second)
 {
@@ -468,7 +499,7 @@ void QOpenGLContext::setScreen(QScreen *screen)
     and/or hardware only supports version 3.2 Core profile contexts then you will
     get a 3.2 Core profile context.
 
-    Returns true if the native context was successfully created and is ready to
+    Returns \c true if the native context was successfully created and is ready to
     be used with makeCurrent(), swapBuffers(), etc.
 
     \sa makeCurrent(), destroy(), format()
@@ -486,7 +517,7 @@ bool QOpenGLContext::create()
         d->shareContext = 0;
     d->shareGroup = d->shareContext ? d->shareContext->shareGroup() : new QOpenGLContextGroup;
     d->shareGroup->d_func()->addContext(this);
-    return d->platformGLContext;
+    return isValid();
 }
 
 /*!
@@ -523,6 +554,8 @@ void QOpenGLContext::destroy()
     d->versionFunctions.clear();
     qDeleteAll(d->versionFunctionsBackend);
     d->versionFunctionsBackend.clear();
+    delete d->textureFunctions;
+    d->textureFunctions = 0;
 }
 
 /*!
@@ -571,6 +604,9 @@ bool QOpenGLContext::isValid() const
     without having to manage it manually.
 
     The context or a sharing context must be current.
+
+    The returned QOpenGLFunctions instance is ready to be used and it
+    does not need initializeOpenGLFunctions() to be called.
 */
 QOpenGLFunctions *QOpenGLContext::functions() const
 {
@@ -583,9 +619,12 @@ QOpenGLFunctions *QOpenGLContext::functions() const
 /*!
     \fn T *QOpenGLContext::versionFunctions() const
 
+    \overload versionFunctions()
+
     Returns a pointer to an object that provides access to all functions for
     the version and profile of this context. Before using any of the functions
-    they must be initialized by calling QAbstractOpenGLFunctions::initializeOpenGLFunctions().
+    they must be initialized by calling QAbstractOpenGLFunctions::initializeOpenGLFunctions()
+    with this context being the current context.
 
     Usually one would use the template version of this function to automatically
     have the result cast to the correct type.
@@ -597,7 +636,7 @@ QOpenGLFunctions *QOpenGLContext::functions() const
             qWarning() << "Could not obtain required OpenGL context version";
             exit(1);
         }
-        funcs->initializeOpenGLFunctions(context);
+        funcs->initializeOpenGLFunctions();
     \endcode
 
     It is possible to request a functions object for a different version and profile
@@ -626,17 +665,23 @@ QOpenGLFunctions *QOpenGLContext::functions() const
 */
 
 /*!
-    Returns a pointer to an object that provides access to all functions for
-    the version of the current context. Before using any of the functions
-    they must be initialized by calling QAbstractOpenGLFunctions::initializeOpenGLFunctions().
+    Returns a pointer to an object that provides access to all functions for the
+    \a versionProfile of this context. Before using any of the functions they must
+    be initialized by calling QAbstractOpenGLFunctions::initializeOpenGLFunctions()
+    with this context being the current context.
 
     Usually one would use the template version of this function to automatically
     have the result cast to the correct type.
-
-    \sa T *QOpenGLContext::versionFunctions()
 */
 QAbstractOpenGLFunctions *QOpenGLContext::versionFunctions(const QOpenGLVersionProfile &versionProfile) const
 {
+#ifndef QT_OPENGL_ES_2
+    if (isOpenGLES()) {
+        qWarning("versionFunctions: Not supported on OpenGL ES");
+        return 0;
+    }
+#endif // QT_OPENGL_ES_2
+
     Q_D(const QOpenGLContext);
     const QSurfaceFormat f = format();
 
@@ -690,8 +735,8 @@ QSet<QByteArray> QOpenGLContext::extensions() const
 }
 
 /*!
-    Returns true if this OpenGL context supports the specified OpenGL
-    \a extension, false otherwise.
+    Returns \c true if this OpenGL context supports the specified OpenGL
+    \a extension, \c false otherwise.
 
     The context or a sharing context must be current.
 
@@ -729,7 +774,7 @@ GLuint QOpenGLContext::defaultFramebufferObject() const
 
 /*!
     Makes the context current in the current thread, against the given
-    \a surface. Returns true if successful.
+    \a surface. Returns \c true if successful.
 
     If \a surface is 0 this is equivalent to calling doneCurrent().
 
@@ -757,8 +802,7 @@ bool QOpenGLContext::makeCurrent(QSurface *surface)
 
     if (!surface->surfaceHandle())
         return false;
-
-    if (surface->surfaceType() != QSurface::OpenGLSurface) {
+    if (!surface->supportsOpenGL()) {
         qWarning() << "QOpenGLContext::makeCurrent() called with non-opengl surface" << surface;
         return false;
     }
@@ -833,9 +877,9 @@ void QOpenGLContext::swapBuffers(QSurface *surface)
         return;
     }
 
-    if (surface->surfaceType() != QSurface::OpenGLSurface) {
-         qWarning() << "QOpenGLContext::swapBuffers() called with non-opengl surface";
-         return;
+    if (!surface->supportsOpenGL()) {
+        qWarning() << "QOpenGLContext::swapBuffers() called with non-opengl surface";
+        return;
     }
 
     if (surface->surfaceClass() == QSurface::Window
@@ -853,7 +897,7 @@ void QOpenGLContext::swapBuffers(QSurface *surface)
         qWarning() << "QOpenGLContext::swapBuffers() called without corresponding makeCurrent()";
 #endif
     if (surface->format().swapBehavior() == QSurfaceFormat::SingleBuffer)
-        glFlush();
+        functions()->glFlush();
     d->platformGLContext->swapBuffers(surfaceHandle);
 }
 
@@ -874,6 +918,20 @@ QFunctionPointer QOpenGLContext::getProcAddress(const QByteArray &procName) cons
     Returns the format of the underlying platform context, if create() has been called.
 
     Otherwise, returns the requested format.
+
+    The requested and the actual format may differ. Requesting a given OpenGL version does
+    not mean the resulting context will target exactly the requested version. It is only
+    guaranteed that the version/profile/options combination for the created context is
+    compatible with the request, as long as the driver is able to provide such a context.
+
+    For example, requesting an OpenGL version 3.x core profile context may result in an
+    OpenGL 4.x core profile context. Similarly, a request for OpenGL 2.1 may result in an
+    OpenGL 3.0 context with deprecated functions enabled. Finally, depending on the
+    driver, unsupported versions may result in either a context creation failure or in a
+    context for the highest supported version.
+
+    Similar differences are possible in the buffer sizes, for example, the resulting
+    context may have a larger depth buffer than requested. This is perfectly normal.
 */
 QSurfaceFormat QOpenGLContext::format() const
 {
@@ -948,6 +1006,85 @@ void QOpenGLContext::deleteQGLContext()
     }
 }
 
+/*!
+  Returns the platform-specific handle for the OpenGL implementation that
+  is currently in use. (for example, a HMODULE on Windows)
+
+  On platforms that do not use dynamic GL switch the return value is null.
+
+  The library might be GL-only, meaning that windowing system interface
+  functions (for example EGL) may live in another, separate library.
+
+  \note This function requires that the QGuiApplication instance is already created.
+
+  \sa openGLModuleType()
+
+  \since 5.3
+ */
+void *QOpenGLContext::openGLModuleHandle()
+{
+#ifdef QT_OPENGL_DYNAMIC
+    QGuiApplication *app = qGuiApp;
+    Q_ASSERT(app);
+    return app->platformNativeInterface()->nativeResourceForIntegration(QByteArrayLiteral("glhandle"));
+#else
+    return 0;
+#endif
+}
+
+/*!
+  \enum QOpenGLContext::OpenGLModuleType
+  This enum defines the type of the underlying OpenGL implementation.
+
+  \value LibGL   OpenGL
+  \value LibGLES OpenGL ES 2.0 or higher
+
+  \since 5.3
+*/
+
+/*!
+  Returns the underlying OpenGL implementation type.
+
+  On platforms where the OpenGL implementation is not dynamically
+  loaded, the return value is determined during compile time and never
+  changes.
+
+  \note A desktop OpenGL implementation may be capable of creating
+  ES-compatible contexts too. Therefore in most cases it is more
+  appropriate to check QSurfaceFormat::renderableType() or using the
+  the convenience function isOpenGLES().
+
+  \note This function requires that the QGuiApplication instance is already created.
+
+  \since 5.3
+ */
+QOpenGLContext::OpenGLModuleType QOpenGLContext::openGLModuleType()
+{
+#if defined(QT_OPENGL_DYNAMIC)
+    Q_ASSERT(qGuiApp);
+    return QGuiApplicationPrivate::instance()->platformIntegration()->openGLModuleType();
+#elif defined(QT_OPENGL_ES_2)
+    return LibGLES;
+#else
+    return LibGL;
+#endif
+}
+
+/*!
+  Returns true if the context is an OpenGL ES context.
+
+  If the context has not yet been created, the result is based on the
+  requested format set via setFormat().
+
+  \sa create(), format(), setFormat()
+
+  \since 5.3
+  */
+bool QOpenGLContext::isOpenGLES() const
+{
+    return format().renderableType() == QSurfaceFormat::OpenGLES;
+}
+
 /*!
     \internal
 */
@@ -976,6 +1113,24 @@ void QOpenGLContext::removeFunctionsBackend(const QOpenGLVersionStatus &v)
     d->versionFunctionsBackend.remove(v);
 }
 
+/*!
+    \internal
+*/
+QOpenGLTextureHelper* QOpenGLContext::textureFunctions() const
+{
+    Q_D(const QOpenGLContext);
+    return d->textureFunctions;
+}
+
+/*!
+    \internal
+*/
+void QOpenGLContext::setTextureFunctions(QOpenGLTextureHelper* textureFuncs)
+{
+    Q_D(QOpenGLContext);
+    d->textureFunctions = textureFuncs;
+}
+
 /*!
     \class QOpenGLContextGroup
     \since 5.0
@@ -1190,7 +1345,8 @@ void QOpenGLSharedResourceGuard::freeResource(QOpenGLContext *context)
     QOpenGLMultiGroupSharedResource instance.
 */
 QOpenGLMultiGroupSharedResource::QOpenGLMultiGroupSharedResource()
-    : active(0)
+    : active(0),
+      m_mutex(QMutex::Recursive)
 {
 #ifdef QT_GL_CONTEXT_RESOURCE_DEBUG
     qDebug("Creating context group resource object %p.", this);
