@@ -79,8 +79,8 @@ QT_BEGIN_NAMESPACE
     QWindow directly. Still, it is possible to render directly to a QWindow
     with QBackingStore or QOpenGLContext, when wanting to keep dependencies to
     a minimum or when wanting to use OpenGL directly. The
-    \l{gui/rasterwindow}{Raster Window} and \l{gui/openglwindow}{OpenGL Window}
-    examples are useful reference examples for how to render to a QWindow using
+    \l{Raster Window Example} and \l{OpenGL Window Example}
+    are useful reference examples for how to render to a QWindow using
     either approach.
 
     \section1 Resource management
@@ -126,8 +126,8 @@ QT_BEGIN_NAMESPACE
     to a window with type QSurface::RasterSurface, and QOpenGLContext for
     rendering with OpenGL to a window with type QSurface::OpenGLSurface.
 
-    The application can start rendering as soon as isExposed() returns true,
-    and can keep rendering until it isExposed() returns false. To find out when
+    The application can start rendering as soon as isExposed() returns \c true,
+    and can keep rendering until it isExposed() returns \c false. To find out when
     isExposed() changes, reimplement exposeEvent(). The window will always get
     a resize event before the first expose event.
 
@@ -159,13 +159,7 @@ QWindow::QWindow(QScreen *targetScreen)
     d->screen = targetScreen;
     if (!d->screen)
         d->screen = QGuiApplication::primaryScreen();
-
-    //if your applications aborts here, then chances are your creating a QWindow before the
-    //screen list is populated.
-    Q_ASSERT(d->screen);
-
-    connect(d->screen, SIGNAL(destroyed(QObject*)), this, SLOT(screenDestroyed(QObject*)));
-    QGuiApplicationPrivate::window_list.prepend(this);
+    d->init();
 }
 
 /*!
@@ -188,8 +182,7 @@ QWindow::QWindow(QWindow *parent)
         d->screen = parent->screen();
     if (!d->screen)
         d->screen = QGuiApplication::primaryScreen();
-    connect(d->screen, SIGNAL(destroyed(QObject*)), this, SLOT(screenDestroyed(QObject*)));
-    QGuiApplicationPrivate::window_list.prepend(this);
+    d->init();
 }
 
 /*!
@@ -214,8 +207,7 @@ QWindow::QWindow(QWindowPrivate &dd, QWindow *parent)
         d->screen = parent->screen();
     if (!d->screen)
         d->screen = QGuiApplication::primaryScreen();
-    connect(d->screen, SIGNAL(destroyed(QObject*)), this, SLOT(screenDestroyed(QObject*)));
-    QGuiApplicationPrivate::window_list.prepend(this);
+    d->init();
 }
 
 /*!
@@ -227,10 +219,26 @@ QWindow::~QWindow()
         QGuiApplicationPrivate::focus_window = 0;
     if (QGuiApplicationPrivate::currentMouseWindow == this)
         QGuiApplicationPrivate::currentMouseWindow = 0;
+    if (QGuiApplicationPrivate::tabletPressTarget == this)
+        QGuiApplicationPrivate::tabletPressTarget = 0;
     QGuiApplicationPrivate::window_list.removeAll(this);
     destroy();
 }
 
+void QWindowPrivate::init()
+{
+    Q_Q(QWindow);
+
+    // If your application aborts here, you are probably creating a QWindow
+    // before the screen list is populated.
+    if (!screen) {
+        qFatal("Cannot create window: no screens available");
+        exit(1);
+    }
+    QObject::connect(screen, SIGNAL(destroyed(QObject*)), q, SLOT(screenDestroyed(QObject*)));
+    QGuiApplicationPrivate::window_list.prepend(q);
+}
+
 /*!
     \enum QWindow::Visibility
     \since 5.1
@@ -348,8 +356,9 @@ void QWindowPrivate::updateVisibility()
 void QWindowPrivate::setScreen(QScreen *newScreen, bool recreate)
 {
     Q_Q(QWindow);
-    if (newScreen != q->screen()) {
-        const bool shouldRecreate = recreate && platformWindow != 0;
+    if (newScreen != screen) {
+        const bool shouldRecreate = recreate && platformWindow != 0
+            && !(screen && screen->virtualSiblings().contains(newScreen));
         if (shouldRecreate)
             q->destroy();
         if (screen)
@@ -364,6 +373,10 @@ void QWindowPrivate::setScreen(QScreen *newScreen, bool recreate)
     }
 }
 
+void QWindowPrivate::clearFocusObject()
+{
+}
+
 /*!
     Sets the \a surfaceType of the window.
 
@@ -422,6 +435,14 @@ void QWindow::setVisible(bool visible)
         // remove posted quit events when showing a new window
         QCoreApplication::removePostedEvents(qApp, QEvent::Quit);
 
+        if (type() == Qt::Window) {
+            QString &firstWindowTitle = QGuiApplicationPrivate::instance()->firstWindowTitle;
+            if (!firstWindowTitle.isEmpty()) {
+                setTitle(firstWindowTitle);
+                firstWindowTitle = QString();
+            }
+        }
+
         QShowEvent showEvent;
         QGuiApplication::sendEvent(this, &showEvent);
     }
@@ -434,7 +455,7 @@ void QWindow::setVisible(bool visible)
     }
 
 #ifndef QT_NO_CURSOR
-    if (visible && d->hasCursor)
+    if (visible && (d->hasCursor || QGuiApplication::overrideCursor()))
         d->applyCursor();
 #endif
     d->platformWindow->setVisible(visible);
@@ -501,10 +522,7 @@ WId QWindow::winId() const
     if(!d->platformWindow)
         const_cast<QWindow *>(this)->create();
 
-    WId id = d->platformWindow->winId();
-    // See the QPlatformWindow::winId() documentation
-    Q_ASSERT(id != WId(0));
-    return id;
+    return d->platformWindow->winId();
 }
 
 /*!
@@ -627,7 +645,7 @@ void QWindow::setFormat(const QSurfaceFormat &format)
 }
 
 /*!
-    Returns the requested surfaceformat of this window.
+    Returns the requested surface format of this window.
 
     If the requested format was not supported by the platform implementation,
     the requestedFormat will differ from the actual window format.
@@ -647,9 +665,17 @@ QSurfaceFormat QWindow::requestedFormat() const
 
     After the window has been created, this function will return the actual surface format
     of the window. It might differ from the requested format if the requested format could
-    not be fulfilled by the platform.
+    not be fulfilled by the platform. It might also be a superset, for example certain
+    buffer sizes may be larger than requested.
 
-    \sa create(), requestedFormat()
+    \note Depending on the platform, certain values in this surface format may still
+    contain the requested values, that is, the values that have been passed to
+    setFormat(). Typical examples are the OpenGL version, profile and options. These may
+    not get updated during create() since these are context specific and a single window
+    may be used together with multiple contexts over its lifetime. Use the
+    QOpenGLContext's format() instead to query such values.
+
+    \sa create(), requestedFormat(), QOpenGLContext::format()
 */
 QSurfaceFormat QWindow::format() const
 {
@@ -712,9 +738,15 @@ Qt::WindowType QWindow::type() const
 void QWindow::setTitle(const QString &title)
 {
     Q_D(QWindow);
-    d->windowTitle = title;
-    if (d->platformWindow)
+    bool changed = false;
+    if (d->windowTitle != title) {
+        d->windowTitle = title;
+        changed = true;
+    }
+    if (d->platformWindow && type() != Qt::Desktop)
         d->platformWindow->setWindowTitle(title);
+    if (changed)
+        emit windowTitleChanged(title);
 }
 
 QString QWindow::title() const
@@ -761,6 +793,8 @@ void QWindow::setIcon(const QIcon &icon)
     d->windowIcon = icon;
     if (d->platformWindow)
         d->platformWindow->setWindowIcon(icon);
+    QEvent e(QEvent::WindowIconChange);
+    QCoreApplication::sendEvent(this, &e);
 }
 
 /*!
@@ -771,6 +805,8 @@ void QWindow::setIcon(const QIcon &icon)
 QIcon QWindow::icon() const
 {
     Q_D(const QWindow);
+    if (d->windowIcon.isNull())
+        return QGuiApplication::windowIcon();
     return d->windowIcon;
 }
 
@@ -871,6 +907,10 @@ QRegion QWindow::mask() const
 void QWindow::requestActivate()
 {
     Q_D(QWindow);
+    if (flags() & Qt::WindowDoesNotAcceptFocus) {
+        qWarning() << "requestActivate() called for " << this << " which has Qt::WindowDoesNotAcceptFocus set.";
+        return;
+    }
     if (d->platformWindow)
         d->platformWindow->requestActivateWindow();
 }
@@ -901,7 +941,7 @@ bool QWindow::isExposed() const
 */
 
 /*!
-    Returns true if the window should appear active from a style perspective.
+    Returns \c true if the window should appear active from a style perspective.
 
     This is the case for the window that has input focus as well as windows
     that are in the same parent / transient parent chain as the focus window.
@@ -954,11 +994,9 @@ void QWindow::reportContentOrientationChange(Qt::ScreenOrientation orientation)
     Q_D(QWindow);
     if (d->contentOrientation == orientation)
         return;
-    if (!d->platformWindow)
-        create();
-    Q_ASSERT(d->platformWindow);
+    if (d->platformWindow)
+        d->platformWindow->handleContentOrientationChange(orientation);
     d->contentOrientation = orientation;
-    d->platformWindow->handleContentOrientationChange(orientation);
     emit contentOrientationChanged(orientation);
 }
 
@@ -980,8 +1018,13 @@ Qt::ScreenOrientation QWindow::contentOrientation() const
 qreal QWindow::devicePixelRatio() const
 {
     Q_D(const QWindow);
+
+    // If there is no platform window, do the second best thing and
+    // return the app global devicePixelRatio. This is the highest
+    // devicePixelRatio found on the system screens, and will be
+    // correct for single-display systems (a very common case).
     if (!d->platformWindow)
-        return 1.0;
+        return qApp->devicePixelRatio();
     return d->platformWindow->devicePixelRatio();
 }
 
@@ -1067,7 +1110,7 @@ QWindow *QWindow::transientParent() const
 */
 
 /*!
-    Returns true if the window is an ancestor of the given \a child. If \a mode
+    Returns \c true if the window is an ancestor of the given \a child. If \a mode
     is IncludeTransients, then transient parents are also considered ancestors.
 */
 bool QWindow::isAncestorOf(const QWindow *child, AncestorMode mode) const
@@ -1640,17 +1683,18 @@ QObject *QWindow::focusObject() const
 /*!
     Shows the window.
 
-    This equivalent to calling showFullScreen() or showNormal(), depending
-    on whether the platform defaults to windows being fullscreen or not, and
-    whether the window is a popup.
+    This is equivalent to calling showFullScreen(), showMaximized(), or showNormal(),
+    depending on the platform's default behavior for the window type and flags.
 
-    \sa showFullScreen(), showNormal(), hide(), QStyleHints::showIsFullScreen(), flags()
+    \sa showFullScreen(), showMaximized(), showNormal(), hide(), QStyleHints::showIsFullScreen(), flags()
 */
 void QWindow::show()
 {
-    bool isPopup = d_func()->windowFlags & Qt::Popup & ~Qt::Window;
-    if (!isPopup && qApp->styleHints()->showIsFullScreen())
+    Qt::WindowState defaultState = QGuiApplicationPrivate::platformIntegration()->defaultWindowState(d_func()->windowFlags);
+    if (defaultState == Qt::WindowFullScreen)
         showFullScreen();
+    else if (defaultState == Qt::WindowMaximized)
+        showMaximized();
     else
         showNormal();
 }
@@ -1707,7 +1751,10 @@ void QWindow::showFullScreen()
 {
     setWindowState(Qt::WindowFullScreen);
     setVisible(true);
+#if !defined Q_OS_QNX // On QNX this window will be activated anyway from libscreen
+                      // activating it here before libscreen activates it causes problems
     requestActivate();
+#endif
 }
 
 /*!
@@ -1728,7 +1775,7 @@ void QWindow::showNormal()
     Close the window.
 
     This closes the window, effectively calling destroy(), and potentially
-    quitting the application. Returns true on success, false if it has a parent
+    quitting the application. Returns \c true on success, false if it has a parent
     window (in which case the top level window should be closed instead).
 
     \sa destroy(), QGuiApplication::quitOnLastWindowClosed()
@@ -1777,7 +1824,7 @@ void QWindow::exposeEvent(QExposeEvent *ev)
 }
 
 /*!
-    Override this to handle mouse events (\a ev).
+    Override this to handle window move events (\a ev).
 */
 void QWindow::moveEvent(QMoveEvent *ev)
 {
@@ -1917,6 +1964,10 @@ bool QWindow::event(QEvent *ev)
         hideEvent(static_cast<QHideEvent *>(ev));
         break;
 
+    case QEvent::ApplicationWindowIconChange:
+        setIcon(icon());
+        break;
+
     case QEvent::WindowStateChange: {
         Q_D(QWindow);
         emit windowStateChanged(d->windowState);
@@ -2135,6 +2186,26 @@ void QWindowPrivate::maybeQuitOnLastWindowClosed()
 
 }
 
+QWindow *QWindowPrivate::topLevelWindow() const
+{
+    Q_Q(const QWindow);
+
+    QWindow *window = const_cast<QWindow *>(q);
+
+    while (window) {
+        QWindow *parent = window->parent();
+        if (!parent)
+            parent = window->transientParent();
+
+        if (!parent)
+            break;
+
+        window = parent;
+    }
+
+    return window;
+}
+
 /*!
     Creates a local representation of a window created by another process or by
     using native libraries below Qt.
