@@ -277,7 +277,7 @@ static bool qt_read_dibv5(QDataStream &s, QImage &image)
 
     int  bpl = image.bytesPerLine();
     uchar *data = image.bits();
-    register QRgb *p;
+    QRgb *p;
     QRgb  *end;
     uchar *buf24 = new uchar[bpl];
     int    bpl24 = ((w*nbits+31)/32)*4;
@@ -469,7 +469,7 @@ int QWindowsMime::registerMimeType(const QString &mime)
 /*!
 \fn bool QWindowsMime::canConvertFromMime(const FORMATETC &formatetc, const QMimeData *mimeData) const
 
-  Returns true if the converter can convert from the \a mimeData to
+  Returns \c true if the converter can convert from the \a mimeData to
   the format specified in \a formatetc.
 
   All subclasses must reimplement this pure virtual function.
@@ -478,7 +478,7 @@ int QWindowsMime::registerMimeType(const QString &mime)
 /*!
   \fn bool QWindowsMime::canConvertToMime(const QString &mimeType, IDataObject *pDataObj) const
 
-  Returns true if the converter can convert to the \a mimeType from
+  Returns \c true if the converter can convert to the \a mimeType from
   the available formats in \a pDataObj.
 
   All subclasses must reimplement this pure virtual function.
@@ -675,8 +675,7 @@ QVariant QWindowsMimeText::convertToMime(const QString &mime, LPDATAOBJECT pData
         else
             ret = str.toUtf8();
     }
-    if (QWindowsContext::verboseOLE)
-        qDebug() << __FUNCTION__ << ret;
+    qCDebug(lcQpaMime) << __FUNCTION__ << ret;
     return ret;
 }
 
@@ -1490,8 +1489,7 @@ QStringList QWindowsMimeConverter::allMimesForFormats(IDataObject *pDataObj) con
         }
         fmtenum->Release();
     }
-    if (QWindowsContext::verboseOLE)
-        qDebug() << __FUNCTION__ << pDataObj << formats;
+    qCDebug(lcQpaMime) << __FUNCTION__ << pDataObj << formats;
     return formats;
 }
 
@@ -1541,9 +1539,8 @@ QVariant QWindowsMimeConverter::convertToMime(const QStringList &mimeTypes,
             if (converter->canConvertToMime(format, pDataObj)) {
                 const QVariant dataV = converter->convertToMime(format, pDataObj, preferredType);
                 if (dataV.isValid()) {
-                    if (QWindowsContext::verboseOLE)
-                        qDebug() << __FUNCTION__ << mimeTypes << "\nFormat: "
-                                 << format << pDataObj << " returns " << dataV;
+                    qCDebug(lcQpaMime) << __FUNCTION__ << mimeTypes << "\nFormat: "
+                        << format << pDataObj << " returns " << dataV;
                     if (formatIn)
                         *formatIn = format;
                     return dataV;
@@ -1551,8 +1548,7 @@ QVariant QWindowsMimeConverter::convertToMime(const QStringList &mimeTypes,
             }
         }
     }
-    if (QWindowsContext::verboseOLE)
-        qDebug() << __FUNCTION__ << "fails" << mimeTypes << pDataObj << preferredType;
+    qCDebug(lcQpaMime) << __FUNCTION__ << "fails" << mimeTypes << pDataObj << preferredType;
     return QVariant();
 }
 
