@@ -51,13 +51,15 @@
 #include "qxcbwmsupport.h"
 #include "qxcbnativeinterface.h"
 #include "qxcbintegration.h"
+#include "qxcbsystemtraytracker.h"
 
-#include <QtAlgorithms>
 #include <QSocketNotifier>
 #include <QAbstractEventDispatcher>
 #include <QTimer>
 #include <QByteArray>
 
+#include <algorithm>
+
 #include <dlfcn.h>
 #include <stdio.h>
 #include <errno.h>
@@ -91,10 +93,37 @@
 QT_BEGIN_NAMESPACE
 
 #ifdef XCB_USE_XLIB
+static const char * const xcbConnectionErrors[] = {
+    "No error", /* Error 0 */
+    "I/O error", /* XCB_CONN_ERROR */
+    "Unsupported extension used", /* XCB_CONN_CLOSED_EXT_NOTSUPPORTED */
+    "Out of memory", /* XCB_CONN_CLOSED_MEM_INSUFFICIENT */
+    "Maximum allowed requested length exceeded", /* XCB_CONN_CLOSED_REQ_LEN_EXCEED */
+    "Failed to parse display string", /* XCB_CONN_CLOSED_PARSE_ERR */
+    "No such screen on display", /* XCB_CONN_CLOSED_INVALID_SCREEN */
+    "Error during FD passing" /* XCB_CONN_CLOSED_FDPASSING_FAILED */
+};
+
 static int nullErrorHandler(Display *, XErrorEvent *)
 {
     return 0;
 }
+
+static int ioErrorHandler(Display *dpy)
+{
+    xcb_connection_t *conn = XGetXCBConnection(dpy);
+    if (conn != NULL) {
+        /* Print a message with a textual description of the error */
+        int code = xcb_connection_has_error(conn);
+        const char *str = "Unknown error";
+        int arrayLength = sizeof(xcbConnectionErrors) / sizeof(xcbConnectionErrors[0]);
+        if (code >= 0 && code < arrayLength)
+            str = xcbConnectionErrors[code];
+
+        qWarning("The X11 connection broke: %s (code %d)", str, code);
+    }
+    return _XDefaultIOError(dpy);
+}
 #endif
 
 QXcbScreen* QXcbConnection::findOrCreateScreen(QList<QXcbScreen *>& newScreens,
@@ -142,23 +171,23 @@ void QXcbConnection::updateScreens()
             xcb_generic_error_t *error = NULL;
             xcb_randr_get_output_primary_cookie_t primaryCookie =
                 xcb_randr_get_output_primary(xcb_connection(), xcbScreen->root);
-            xcb_randr_get_screen_resources_current_cookie_t resourcesCookie =
-                xcb_randr_get_screen_resources_current(xcb_connection(), xcbScreen->root);
+            xcb_randr_get_screen_resources_cookie_t resourcesCookie =
+                xcb_randr_get_screen_resources(xcb_connection(), xcbScreen->root);
             xcb_randr_get_output_primary_reply_t *primary =
                     xcb_randr_get_output_primary_reply(xcb_connection(), primaryCookie, &error);
             if (!primary || error) {
                 qWarning("QXcbConnection: Failed to get the primary output of the screen");
                 free(error);
             } else {
-                xcb_randr_get_screen_resources_current_reply_t *resources =
-                        xcb_randr_get_screen_resources_current_reply(xcb_connection(), resourcesCookie, &error);
+                xcb_randr_get_screen_resources_reply_t *resources =
+                        xcb_randr_get_screen_resources_reply(xcb_connection(), resourcesCookie, &error);
                 if (!resources || error) {
                     qWarning("QXcbConnection: Failed to get the screen resources");
                     free(error);
                 } else {
                     xcb_timestamp_t timestamp = resources->config_timestamp;
-                    outputCount = xcb_randr_get_screen_resources_current_outputs_length(resources);
-                    xcb_randr_output_t *outputs = xcb_randr_get_screen_resources_current_outputs(resources);
+                    outputCount = xcb_randr_get_screen_resources_outputs_length(resources);
+                    xcb_randr_output_t *outputs = xcb_randr_get_screen_resources_outputs(resources);
 
                     for (int i = 0; i < outputCount; i++) {
                         xcb_randr_get_output_info_reply_t *output =
@@ -183,8 +212,12 @@ void QXcbConnection::updateScreens()
                         siblings << screen;
                         activeScreens << screen;
                         ++screenNumber;
-                        if (!primaryScreen && primary) {
-                            if (m_primaryScreen == xcbScreenNumber && (primary->output == XCB_NONE || outputs[i] == primary->output)) {
+                        // There can be multiple outputs per screen, use either
+                        // the first or an exact match.  An exact match isn't
+                        // always available if primary->output is XCB_NONE
+                        // or currently disconnected output.
+                        if (m_primaryScreen == xcbScreenNumber) {
+                            if (!primaryScreen || (primary && outputs[i] == primary->output)) {
                                 primaryScreen = screen;
                                 siblings.prepend(siblings.takeLast());
 #ifdef Q_XCB_DEBUG
@@ -260,24 +293,26 @@ QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGra
     , has_input_shape(false)
     , has_touch_without_mouse_emulation(false)
     , has_xkb(false)
+    , debug_xinput_devices(false)
+    , debug_xinput(false)
     , m_buttons(0)
     , m_focusWindow(0)
+    , m_systemTrayTracker(0)
 {
+#ifdef XCB_USE_EGL
+    EGLNativeDisplayType dpy = EGL_DEFAULT_DISPLAY;
+#elif defined(XCB_USE_XLIB)
+    Display *dpy;
+#endif
 #ifdef XCB_USE_XLIB
-    Display *dpy = XOpenDisplay(m_displayName.constData());
+    dpy = XOpenDisplay(m_displayName.constData());
     if (dpy) {
         m_primaryScreen = DefaultScreen(dpy);
         m_connection = XGetXCBConnection(dpy);
         XSetEventQueueOwner(dpy, XCBOwnsEventQueue);
         XSetErrorHandler(nullErrorHandler);
+        XSetIOErrorHandler(ioErrorHandler);
         m_xlib_display = dpy;
-#ifdef XCB_USE_EGL
-        EGLDisplay eglDisplay = eglGetDisplay(dpy);
-        m_egl_display = eglDisplay;
-        EGLint major, minor;
-        eglBindAPI(EGL_OPENGL_ES_API);
-        m_has_egl = eglInitialize(eglDisplay,&major,&minor);
-#endif //XCB_USE_EGL
     }
 #else
     m_connection = xcb_connect(m_displayName.constData(), &m_primaryScreen);
@@ -286,17 +321,15 @@ QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGra
     if (!m_connection || xcb_connection_has_error(m_connection))
         qFatal("QXcbConnection: Could not connect to display %s", m_displayName.constData());
 
-    m_reader = new QXcbEventReader(this);
-    connect(m_reader, SIGNAL(eventPending()), this, SLOT(processXcbEvents()), Qt::QueuedConnection);
-    connect(m_reader, SIGNAL(finished()), this, SLOT(processXcbEvents()));
-    if (!m_reader->startThread()) {
-        QSocketNotifier *notifier = new QSocketNotifier(xcb_get_file_descriptor(xcb_connection()), QSocketNotifier::Read, this);
-        connect(notifier, SIGNAL(activated(int)), this, SLOT(processXcbEvents()));
+#ifdef XCB_USE_EGL
+    EGLDisplay eglDisplay = eglGetDisplay(dpy);
+    m_egl_display = eglDisplay;
+    EGLint major, minor;
+    m_has_egl = eglInitialize(eglDisplay, &major, &minor);
+#endif //XCB_USE_EGL
 
-        QAbstractEventDispatcher *dispatcher = QGuiApplicationPrivate::eventDispatcher;
-        connect(dispatcher, SIGNAL(aboutToBlock()), this, SLOT(processXcbEvents()));
-        connect(dispatcher, SIGNAL(awake()), this, SLOT(processXcbEvents()));
-    }
+    m_reader = new QXcbEventReader(this);
+    m_reader->start();
 
     xcb_extension_t *extensions[] = {
         &xcb_shm_id, &xcb_xfixes_id, &xcb_randr_id, &xcb_shape_id, &xcb_sync_id,
@@ -325,12 +358,6 @@ QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGra
     initializeXRandr();
     updateScreens();
 
-    m_connectionEventListener = xcb_generate_id(m_connection);
-    xcb_create_window(m_connection, XCB_COPY_FROM_PARENT,
-                      m_connectionEventListener, m_screens.at(0)->root(),
-                      0, 0, 1, 1, 0, XCB_WINDOW_CLASS_INPUT_ONLY,
-                      m_screens.at(0)->screen()->root_visual, 0, 0);
-
     initializeGLX();
     initializeXFixes();
     initializeXRender();
@@ -367,9 +394,6 @@ QXcbConnection::~QXcbConnection()
 #ifndef QT_NO_DRAGANDDROP
     delete m_drag;
 #endif
-    // Delete screens in reverse order to avoid crash in case of multiple screens
-    while (!m_screens.isEmpty())
-        delete m_screens.takeLast();
 
 #ifdef XCB_USE_XINPUT2_MAEMO
     finalizeXInput2Maemo();
@@ -384,6 +408,10 @@ QXcbConnection::~QXcbConnection()
 
     delete m_reader;
 
+    // Delete screens in reverse order to avoid crash in case of multiple screens
+    while (!m_screens.isEmpty())
+        delete m_screens.takeLast();
+
 #ifdef XCB_USE_EGL
     if (m_has_egl)
         eglTerminate(m_egl_display);
@@ -656,6 +684,11 @@ void QXcbConnection::log(const char *file, int line, int sequence)
 
 void QXcbConnection::handleXcbError(xcb_generic_error_t *error)
 {
+    long result = 0;
+    QAbstractEventDispatcher* dispatcher = QAbstractEventDispatcher::instance();
+    if (dispatcher && dispatcher->filterNativeEvent(m_nativeInterface->genericEventFilterType(), error, &result))
+        return;
+
     uint clamped_error_code = qMin<uint>(error->error_code, (sizeof(xcb_errors) / sizeof(xcb_errors[0])) - 1);
     uint clamped_major_code = qMin<uint>(error->major_code, (sizeof(xcb_protocol_request_codes) / sizeof(xcb_protocol_request_codes[0])) - 1);
 
@@ -738,6 +771,8 @@ void QXcbConnection::handleButtonPress(xcb_generic_event_t *ev)
     // the rest we need to manage ourselves
     m_buttons = (m_buttons & ~0x7) | translateMouseButtons(event->state);
     m_buttons |= translateMouseButton(event->detail);
+    if (Q_UNLIKELY(debug_xinput))
+        qDebug("xcb: pressed mouse button %d, button state %X", event->detail, static_cast<unsigned int>(m_buttons));
 }
 
 void QXcbConnection::handleButtonRelease(xcb_generic_event_t *ev)
@@ -748,6 +783,8 @@ void QXcbConnection::handleButtonRelease(xcb_generic_event_t *ev)
     // the rest we need to manage ourselves
     m_buttons = (m_buttons & ~0x7) | translateMouseButtons(event->state);
     m_buttons &= ~translateMouseButton(event->detail);
+    if (Q_UNLIKELY(debug_xinput))
+        qDebug("xcb: released mouse button %d, button state %X", event->detail, static_cast<unsigned int>(m_buttons));
 }
 
 #ifndef QT_NO_XKB
@@ -761,6 +798,7 @@ namespace {
             xcb_timestamp_t time;
             uint8_t deviceID;
         } any;
+        xcb_xkb_new_keyboard_notify_event_t new_keyboard_notify;
         xcb_xkb_map_notify_event_t map_notify;
         xcb_xkb_state_notify_event_t state_notify;
     } _xkb_event;
@@ -791,21 +829,19 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         case XCB_EXPOSE:
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_expose_event_t, window, handleExposeEvent);
         case XCB_BUTTON_PRESS:
-#ifdef QT_NO_XKB
             m_keyboard->updateXKBStateFromCore(((xcb_button_press_event_t *)event)->state);
-#endif
             handleButtonPress(event);
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_button_press_event_t, event, handleButtonPressEvent);
         case XCB_BUTTON_RELEASE:
-#ifdef QT_NO_XKB
             m_keyboard->updateXKBStateFromCore(((xcb_button_release_event_t *)event)->state);
-#endif
             handleButtonRelease(event);
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_button_release_event_t, event, handleButtonReleaseEvent);
         case XCB_MOTION_NOTIFY:
-#ifdef QT_NO_XKB
+            if (Q_UNLIKELY(debug_xinput)) {
+                xcb_motion_notify_event_t *mev = (xcb_motion_notify_event_t *)event;
+                qDebug("xcb: moved mouse to %4d, %4d; button state %X", mev->event_x, mev->event_y, static_cast<unsigned int>(m_buttons));
+            }
             m_keyboard->updateXKBStateFromCore(((xcb_motion_notify_event_t *)event)->state);
-#endif
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_motion_notify_event_t, event, handleMotionNotifyEvent);
         case XCB_CONFIGURE_NOTIFY:
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_configure_notify_event_t, event, handleConfigureNotifyEvent);
@@ -813,35 +849,29 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_map_notify_event_t, event, handleMapNotifyEvent);
         case XCB_UNMAP_NOTIFY:
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_unmap_notify_event_t, event, handleUnmapNotifyEvent);
+        case XCB_DESTROY_NOTIFY:
+            HANDLE_PLATFORM_WINDOW_EVENT(xcb_destroy_notify_event_t, event, handleDestroyNotifyEvent);
         case XCB_CLIENT_MESSAGE:
             handleClientMessageEvent((xcb_client_message_event_t *)event);
             break;
         case XCB_ENTER_NOTIFY:
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_enter_notify_event_t, event, handleEnterNotifyEvent);
         case XCB_LEAVE_NOTIFY:
-#ifdef QT_NO_XKB
             m_keyboard->updateXKBStateFromCore(((xcb_leave_notify_event_t *)event)->state);
-#endif
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_leave_notify_event_t, event, handleLeaveNotifyEvent);
         case XCB_FOCUS_IN:
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_focus_in_event_t, event, handleFocusInEvent);
         case XCB_FOCUS_OUT:
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_focus_out_event_t, event, handleFocusOutEvent);
         case XCB_KEY_PRESS:
-#ifdef QT_NO_XKB
             m_keyboard->updateXKBStateFromCore(((xcb_key_press_event_t *)event)->state);
-#endif
             HANDLE_KEYBOARD_EVENT(xcb_key_press_event_t, handleKeyPressEvent);
         case XCB_KEY_RELEASE:
-#ifdef QT_NO_XKB
             m_keyboard->updateXKBStateFromCore(((xcb_key_release_event_t *)event)->state);
-#endif
             HANDLE_KEYBOARD_EVENT(xcb_key_release_event_t, handleKeyReleaseEvent);
-#ifdef QT_NO_XKB
         case XCB_MAPPING_NOTIFY:
             m_keyboard->handleMappingNotifyEvent((xcb_mapping_notify_event_t *)event);
             break;
-#endif
         case XCB_SELECTION_REQUEST:
         {
             xcb_selection_request_event_t *sr = (xcb_selection_request_event_t *)event;
@@ -909,6 +939,8 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
             _xkb_event *xkb_event = reinterpret_cast<_xkb_event *>(event);
             if (xkb_event->any.deviceID == m_keyboard->coreDeviceId()) {
                 switch (xkb_event->any.xkbType) {
+                    // XkbNewKkdNotify and XkbMapNotify together capture all sorts of keymap
+                    // updates (e.g. xmodmap, xkbcomp, setxkbmap), with minimal redundent recompilations.
                     case XCB_XKB_STATE_NOTIFY:
                         m_keyboard->updateXKBState(&xkb_event->state_notify);
                         handled = true;
@@ -917,6 +949,12 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
                         m_keyboard->handleMappingNotifyEvent(&xkb_event->map_notify);
                         handled = true;
                         break;
+                    case XCB_XKB_NEW_KEYBOARD_NOTIFY: {
+                        xcb_xkb_new_keyboard_notify_event_t *ev = &xkb_event->new_keyboard_notify;
+                        if (ev->changed & XCB_XKB_NKN_DETAIL_KEYCODES)
+                            m_keyboard->updateKeymap();
+                        break;
+                    }
                     default:
                         break;
                 }
@@ -967,14 +1005,36 @@ QXcbEventReader::QXcbEventReader(QXcbConnection *connection)
 #endif
 }
 
-bool QXcbEventReader::startThread()
+void QXcbEventReader::start()
 {
     if (m_xcb_poll_for_queued_event) {
+        connect(this, SIGNAL(eventPending()), m_connection, SLOT(processXcbEvents()), Qt::QueuedConnection);
+        connect(this, SIGNAL(finished()), m_connection, SLOT(processXcbEvents()));
         QThread::start();
-        return true;
+    } else {
+        // Must be done after we have an event-dispatcher. By posting a method invocation
+        // we are sure that by the time the method is called we have an event-dispatcher.
+        QMetaObject::invokeMethod(this, "registerForEvents", Qt::QueuedConnection);
     }
+}
 
-    return false;
+void QXcbEventReader::registerForEvents()
+{
+    QSocketNotifier *notifier = new QSocketNotifier(xcb_get_file_descriptor(m_connection->xcb_connection()), QSocketNotifier::Read, this);
+    connect(notifier, SIGNAL(activated(int)), m_connection, SLOT(processXcbEvents()));
+
+    QAbstractEventDispatcher *dispatcher = QGuiApplicationPrivate::eventDispatcher;
+    connect(dispatcher, SIGNAL(aboutToBlock()), m_connection, SLOT(processXcbEvents()));
+    connect(dispatcher, SIGNAL(awake()), m_connection, SLOT(processXcbEvents()));
+}
+
+void QXcbEventReader::registerEventDispatcher(QAbstractEventDispatcher *dispatcher)
+{
+    // flush the xcb connection before the EventDispatcher is going to block
+    // In the non-threaded case processXcbEvents is called before going to block,
+    // which flushes the connection.
+    if (m_xcb_poll_for_queued_event)
+        connect(dispatcher, SIGNAL(aboutToBlock()), m_connection, SLOT(flush()));
 }
 
 void QXcbEventReader::run()
@@ -989,8 +1049,11 @@ void QXcbEventReader::run()
         emit eventPending();
     }
 
+    m_mutex.lock();
     for (int i = 0; i < m_events.size(); ++i)
         free(m_events.at(i));
+    m_events.clear();
+    m_mutex.unlock();
 }
 
 void QXcbEventReader::addEvent(xcb_generic_event_t *event)
@@ -1038,14 +1101,21 @@ void QXcbConnection::sendConnectionEvent(QXcbAtom::Atom a, uint id)
     xcb_client_message_event_t event;
     memset(&event, 0, sizeof(event));
 
+    const xcb_window_t eventListener = xcb_generate_id(m_connection);
+    Q_XCB_CALL(xcb_create_window(m_connection, XCB_COPY_FROM_PARENT,
+                                 eventListener, m_screens.at(0)->root(),
+                                 0, 0, 1, 1, 0, XCB_WINDOW_CLASS_INPUT_ONLY,
+                                 m_screens.at(0)->screen()->root_visual, 0, 0));
+
     event.response_type = XCB_CLIENT_MESSAGE;
     event.format = 32;
     event.sequence = 0;
-    event.window = m_connectionEventListener;
+    event.window = eventListener;
     event.type = atom(a);
     event.data.data32[0] = id;
 
-    Q_XCB_CALL(xcb_send_event(xcb_connection(), false, m_connectionEventListener, XCB_EVENT_MASK_NO_EVENT, (const char *)&event));
+    Q_XCB_CALL(xcb_send_event(xcb_connection(), false, eventListener, XCB_EVENT_MASK_NO_EVENT, (const char *)&event));
+    Q_XCB_CALL(xcb_destroy_window(m_connection, eventListener));
     xcb_flush(xcb_connection());
 }
 
@@ -1120,6 +1190,7 @@ void QXcbConnection::processXcbEvents()
         xcb_generic_event_t *event = eventqueue->at(i);
         if (!event)
             continue;
+        QScopedPointer<xcb_generic_event_t, QScopedPointerPodDeleter> eventGuard(event);
         (*eventqueue)[i] = 0;
 
         uint response_type = event->response_type & ~0x80;
@@ -1151,6 +1222,12 @@ void QXcbConnection::processXcbEvents()
                     continue;
             }
 
+            bool accepted = false;
+            if (clipboard()->processIncr())
+                clipboard()->incrTransactionPeeker(event, accepted);
+            if (accepted)
+                continue;
+
             QVector<PeekFunc>::iterator it = m_peekFuncs.begin();
             while (it != m_peekFuncs.end()) {
                 // These callbacks return true if the event is what they were
@@ -1164,8 +1241,6 @@ void QXcbConnection::processXcbEvents()
             handleXcbEvent(event);
             m_reader->lock();
         }
-
-        free(event);
     }
 
     eventqueue->clear();
@@ -1193,6 +1268,8 @@ void QXcbConnection::handleClientMessageEvent(const xcb_client_message_event_t *
         drag()->handleFinished(event);
     }
 #endif
+    if (m_systemTrayTracker && event->type == atom(QXcbAtom::MANAGER))
+        m_systemTrayTracker->notifyManagerClientMessageEvent(event);
 
     QXcbWindow *window = platformWindowFromId(event->window);
     if (!window)
@@ -1228,10 +1305,13 @@ static const char * xcb_atomnames = {
     "_NET_WM_CONTEXT_HELP\0"
     "_NET_WM_SYNC_REQUEST\0"
     "_NET_WM_SYNC_REQUEST_COUNTER\0"
+    "MANAGER\0"
+    "_NET_SYSTEM_TRAY_OPCODE\0"
 
     // ICCCM window state
     "WM_STATE\0"
     "WM_CHANGE_STATE\0"
+    "WM_CLASS\0"
 
     // Session management
     "WM_CLIENT_LEADER\0"
@@ -1384,6 +1464,8 @@ static const char * xcb_atomnames = {
     "Abs MT Pressure\0"
     "Abs MT Tracking ID\0"
     "Max Contacts\0"
+    "Rel X\0"
+    "Rel Y\0"
     // XInput2 tablet
     "Abs X\0"
     "Abs Y\0"
@@ -1394,15 +1476,22 @@ static const char * xcb_atomnames = {
     "Abs Distance\0"
     "Wacom Serial IDs\0"
     "INTEGER\0"
+    "Rel Horiz Wheel\0"
+    "Rel Vert Wheel\0"
+    "Rel Horiz Scroll\0"
+    "Rel Vert Scroll\0"
 #if XCB_USE_MAEMO_WINDOW_PROPERTIES
     "_MEEGOTOUCH_ORIENTATION_ANGLE\0"
 #endif
-    "_XSETTINGS_SETTINGS"
+    "_XSETTINGS_SETTINGS\0"
+    "_COMPIZ_DECOR_PENDING\0"
+    "_COMPIZ_DECOR_REQUEST\0"
+    "_COMPIZ_DECOR_DELETE_PIXMAP\0" // \0\0 terminates loop.
 };
 
-xcb_atom_t QXcbConnection::atom(QXcbAtom::Atom atom)
+QXcbAtom::Atom QXcbConnection::qatom(xcb_atom_t xatom) const
 {
-    return m_allAtoms[atom];
+    return static_cast<QXcbAtom::Atom>(std::find(m_allAtoms, m_allAtoms + QXcbAtom::NAtoms, xatom) - m_allAtoms);
 }
 
 void QXcbConnection::initializeAllAtoms() {
@@ -1608,6 +1697,7 @@ void QXcbConnection::initializeXKB()
 #ifndef QT_NO_XKB
     const xcb_query_extension_reply_t *reply = xcb_get_extension_data(m_connection, &xcb_xkb_id);
     if (!reply || !reply->present) {
+        qWarning() << "Qt: XKEYBOARD extension not present on the X server.";
         xkb_first_event = 0;
         return;
     }
@@ -1617,14 +1707,14 @@ void QXcbConnection::initializeXKB()
     xcb_xkb_use_extension_cookie_t xkb_query_cookie;
     xcb_xkb_use_extension_reply_t *xkb_query;
 
-    xkb_query_cookie = xcb_xkb_use_extension(c, XCB_XKB_MAJOR_VERSION, XCB_XKB_MINOR_VERSION);
+    xkb_query_cookie = xcb_xkb_use_extension(c, XKB_X11_MIN_MAJOR_XKB_VERSION, XKB_X11_MIN_MINOR_XKB_VERSION);
     xkb_query = xcb_xkb_use_extension_reply(c, xkb_query_cookie, 0);
 
     if (!xkb_query) {
         qWarning("Qt: Failed to initialize XKB extension");
         return;
     } else if (!xkb_query->supported) {
-        qWarning("Qt: Unsupported XKB version (want %d %d, has %d %d)",
+        qWarning("Qt: Unsupported XKB version (We want %d %d, but X server has %d %d)",
                  XCB_XKB_MAJOR_VERSION, XCB_XKB_MINOR_VERSION,
                  xkb_query->serverMajor, xkb_query->serverMinor);
         free(xkb_query);
@@ -1634,25 +1724,28 @@ void QXcbConnection::initializeXKB()
     has_xkb = true;
     free(xkb_query);
 
-    uint affectMap, map;
-    affectMap = map = XCB_XKB_MAP_PART_KEY_TYPES |
-            XCB_XKB_MAP_PART_KEY_SYMS |
-            XCB_XKB_MAP_PART_MODIFIER_MAP |
-            XCB_XKB_MAP_PART_EXPLICIT_COMPONENTS |
-            XCB_XKB_MAP_PART_KEY_ACTIONS |
-            XCB_XKB_MAP_PART_KEY_BEHAVIORS |
-            XCB_XKB_MAP_PART_VIRTUAL_MODS |
-            XCB_XKB_MAP_PART_VIRTUAL_MOD_MAP;
+    const uint16_t required_map_parts = (XCB_XKB_MAP_PART_KEY_TYPES |
+        XCB_XKB_MAP_PART_KEY_SYMS |
+        XCB_XKB_MAP_PART_MODIFIER_MAP |
+        XCB_XKB_MAP_PART_EXPLICIT_COMPONENTS |
+        XCB_XKB_MAP_PART_KEY_ACTIONS |
+        XCB_XKB_MAP_PART_KEY_BEHAVIORS |
+        XCB_XKB_MAP_PART_VIRTUAL_MODS |
+        XCB_XKB_MAP_PART_VIRTUAL_MOD_MAP);
 
-    // Xkb events are reported to all interested clients without regard
+    const uint16_t required_events = (XCB_XKB_EVENT_TYPE_NEW_KEYBOARD_NOTIFY |
+        XCB_XKB_EVENT_TYPE_MAP_NOTIFY |
+        XCB_XKB_EVENT_TYPE_STATE_NOTIFY);
+
+    // XKB events are reported to all interested clients without regard
     // to the current keyboard input focus or grab state
     xcb_void_cookie_t select = xcb_xkb_select_events_checked(c,
                        XCB_XKB_ID_USE_CORE_KBD,
-                       XCB_XKB_EVENT_TYPE_STATE_NOTIFY | XCB_XKB_EVENT_TYPE_MAP_NOTIFY,
+                       required_events,
                        0,
-                       XCB_XKB_EVENT_TYPE_STATE_NOTIFY | XCB_XKB_EVENT_TYPE_MAP_NOTIFY,
-                       affectMap,
-                       map,
+                       required_events,
+                       required_map_parts,
+                       required_map_parts,
                        0);
 
     xcb_generic_error_t *error = xcb_request_check(c, select);
@@ -1672,21 +1765,23 @@ bool QXcbConnection::hasEgl() const
 #endif // defined(XCB_USE_EGL)
 
 #if defined(XCB_USE_XINPUT2) || defined(XCB_USE_XINPUT2_MAEMO)
-// Borrowed from libXi.
-int QXcbConnection::xi2CountBits(unsigned char *ptr, int len)
+static int xi2ValuatorOffset(unsigned char *maskPtr, int maskLen, int number)
 {
-    int bits = 0;
-    int i;
-    unsigned char x;
-
-    for (i = 0; i < len; i++) {
-        x = ptr[i];
-        while (x > 0) {
-            bits += (x & 0x1);
-            x >>= 1;
+    int offset = 0;
+    for (int i = 0; i < maskLen; i++) {
+        if (number < 8) {
+            if ((maskPtr[i] & (1 << number)) == 0)
+                return -1;
         }
+        for (int j = 0; j < 8; j++) {
+            if (j == number)
+                return offset;
+            if (maskPtr[i] & (1 << j))
+                offset++;
+        }
+        number -= 8;
     }
-    return bits;
+    return -1;
 }
 
 bool QXcbConnection::xi2GetValuatorValueIfSet(void *event, int valuatorNum, double *value)
@@ -1695,20 +1790,36 @@ bool QXcbConnection::xi2GetValuatorValueIfSet(void *event, int valuatorNum, doub
     unsigned char *buttonsMaskAddr = (unsigned char*)&xideviceevent[1];
     unsigned char *valuatorsMaskAddr = buttonsMaskAddr + xideviceevent->buttons_len * 4;
     FP3232 *valuatorsValuesAddr = (FP3232*)(valuatorsMaskAddr + xideviceevent->valuators_len * 4);
-    int numValuatorValues = xi2CountBits(valuatorsMaskAddr, xideviceevent->valuators_len * 4);
-    // This relies on all bit being set until a certain number i.e. it doesn't support only bit 0 and 5 being set in the mask.
-    // Just like the original code, works for now.
-    if (valuatorNum >= numValuatorValues)
+
+    int valuatorOffset = xi2ValuatorOffset(valuatorsMaskAddr, xideviceevent->valuators_len, valuatorNum);
+    if (valuatorOffset < 0)
         return false;
-    *value = valuatorsValuesAddr[valuatorNum].integral;
-    *value += ((double)valuatorsValuesAddr[valuatorNum].frac / (1 << 16) / (1 << 16));
+
+    *value = valuatorsValuesAddr[valuatorOffset].integral;
+    *value += ((double)valuatorsValuesAddr[valuatorOffset].frac / (1 << 16) / (1 << 16));
     return true;
 }
 
-bool QXcbConnection::xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event, int opCode)
+// Starting from the xcb version 1.9.3 struct xcb_ge_event_t has changed:
+// - "pad0" became "extension"
+// - "pad1" and "pad" became "pad0"
+// New and old version of this struct share the following fields:
+// NOTE: API might change again in the next release of xcb in which case this comment will
+// need to be updated to reflect the reality.
+typedef struct qt_xcb_ge_event_t {
+    uint8_t  response_type;
+    uint8_t  extension;
+    uint16_t sequence;
+    uint32_t length;
+    uint16_t event_type;
+} qt_xcb_ge_event_t;
+
+bool QXcbConnection::xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *ev, int opCode)
 {
-    // xGenericEvent has "extension" on the second byte, xcb_ge_event_t has "pad0".
-    if (event->pad0 == opCode) {
+    qt_xcb_ge_event_t *event = (qt_xcb_ge_event_t *)ev;
+    // xGenericEvent has "extension" on the second byte, the same is true for xcb_ge_event_t starting from
+    // the xcb version 1.9.3, prior to that it was called "pad0".
+    if (event->extension == opCode) {
         // xcb event structs contain stuff that wasn't on the wire, the full_sequence field
         // adds an extra 4 bytes and generic events cookie data is on the wire right after the standard 32 bytes.
         // Move this data back to have the same layout in memory as it was on the wire
@@ -1720,6 +1831,22 @@ bool QXcbConnection::xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event, int o
 }
 #endif // defined(XCB_USE_XINPUT2) || defined(XCB_USE_XINPUT2_MAEMO)
 
+QXcbSystemTrayTracker *QXcbConnection::systemTrayTracker()
+{
+    if (!m_systemTrayTracker) {
+        if ( (m_systemTrayTracker = QXcbSystemTrayTracker::create(this)) ) {
+            connect(m_systemTrayTracker, SIGNAL(systemTrayWindowChanged(QScreen*)),
+                    QGuiApplication::platformNativeInterface(), SIGNAL(systemTrayWindowChanged(QScreen*)));
+        }
+    }
+    return m_systemTrayTracker;
+}
+
+void QXcbConnection::syncWindow(QXcbWindow *window)
+{
+    window->updateSyncRequestCounter();
+}
+
 QXcbConnectionGrabber::QXcbConnectionGrabber(QXcbConnection *connection)
     :m_connection(connection)
 {
