@@ -45,6 +45,8 @@
 #include <qicon.h>
 #include <qiconengine.h>
 
+#include <algorithm>
+
 
 class tst_QIcon : public QObject
 {
@@ -375,32 +377,32 @@ void tst_QIcon::detach()
 void tst_QIcon::addFile()
 {
     QIcon icon;
-    icon.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png"));
-    icon.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-32.png"));
-    icon.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-128.png"));
-    icon.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-16.png"), QSize(), QIcon::Selected);
-    icon.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-32.png"), QSize(), QIcon::Selected);
-    icon.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-128.png"), QSize(), QIcon::Selected);
+    icon.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png"));
+    icon.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-open-32.png"));
+    icon.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-open-128.png"));
+    icon.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-save-16.png"), QSize(), QIcon::Selected);
+    icon.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-save-32.png"), QSize(), QIcon::Selected);
+    icon.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-save-128.png"), QSize(), QIcon::Selected);
 
 #ifndef Q_OS_WINCE
     QVERIFY(icon.pixmap(16, QIcon::Normal).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png")).toImage());
     QVERIFY(icon.pixmap(32, QIcon::Normal).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-32.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-open-32.png")).toImage());
     QVERIFY(icon.pixmap(128, QIcon::Normal).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-128.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-open-128.png")).toImage());
     QVERIFY(icon.pixmap(16, QIcon::Selected).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-16.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-save-16.png")).toImage());
     QVERIFY(icon.pixmap(32, QIcon::Selected).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-32.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-save-32.png")).toImage());
     QVERIFY(icon.pixmap(128, QIcon::Selected).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-128.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-save-128.png")).toImage());
 #else
     // WinCE only includes the 16x16 images for size reasons
     QVERIFY(icon.pixmap(16, QIcon::Normal).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png")).toImage());
     QVERIFY(icon.pixmap(16, QIcon::Selected).toImage() ==
-            QPixmap(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-save-16.png")).toImage());
+            QPixmap(QLatin1String(":/styles/commonstyle/images/standardbutton-save-16.png")).toImage());
 #endif
 }
 
@@ -421,7 +423,7 @@ void tst_QIcon::availableSizes()
 
         QList<QSize> availableSizes = icon.availableSizes();
         QCOMPARE(availableSizes.size(), 3);
-        qSort(availableSizes.begin(), availableSizes.end(), sizeLess);
+        std::sort(availableSizes.begin(), availableSizes.end(), sizeLess);
         QCOMPARE(availableSizes.at(0), QSize(32,32));
         QCOMPARE(availableSizes.at(1), QSize(64,64));
         QCOMPARE(availableSizes.at(2), QSize(128,128));
@@ -437,7 +439,7 @@ void tst_QIcon::availableSizes()
 
     {
         // we try to load an icon from resources
-        QIcon icon(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png"));
+        QIcon icon(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png"));
         QList<QSize> availableSizes = icon.availableSizes();
         QCOMPARE(availableSizes.size(), 1);
         QCOMPARE(availableSizes.at(0), QSize(16, 16));
@@ -446,7 +448,7 @@ void tst_QIcon::availableSizes()
     {
         // load an icon from binary data.
         QPixmap pix;
-        QFile file(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png"));
+        QFile file(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png"));
         QVERIFY(file.open(QIODevice::ReadOnly));
         uchar *data = file.map(0, file.size());
         QVERIFY(data != 0);
@@ -594,7 +596,9 @@ void tst_QIcon::fromTheme()
     QCOMPARE(appointmentIcon.pixmap(22).size(), QSize(22, 22)); // exact
     QCOMPARE(appointmentIcon.pixmap(32).size(), QSize(32, 32)); // exact
     QCOMPARE(appointmentIcon.pixmap(48).size(), QSize(32, 32)); // smaller
+    QCOMPARE(appointmentIcon.pixmap(16).size(), QSize(16, 16)); // scaled down
     QCOMPARE(appointmentIcon.pixmap(8).size(), QSize(8, 8)); // scaled down
+    QCOMPARE(appointmentIcon.pixmap(16).size(), QSize(16, 16)); // scaled down
 
     QByteArray ba;
     // write to QByteArray
@@ -626,12 +630,12 @@ void tst_QIcon::fromTheme()
 void tst_QIcon::task223279_inconsistentAddFile()
 {
     QIcon icon1;
-    icon1.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png"));
+    icon1.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png"));
     icon1.addFile(QLatin1String("IconThatDoesntExist"), QSize(32, 32));
     QPixmap pm1 = icon1.pixmap(32, 32);
 
     QIcon icon2;
-    icon2.addFile(QLatin1String(":/qt-project.org/styles/commonstyle/images/standardbutton-open-16.png"));
+    icon2.addFile(QLatin1String(":/styles/commonstyle/images/standardbutton-open-16.png"));
     icon2.addFile(QLatin1String("IconThatDoesntExist"));
     QPixmap pm2 = icon1.pixmap(32, 32);
 
