@@ -45,7 +45,9 @@
 #include "qserialport.h"
 #include "qserialportinfo.h"
 
-#ifdef Q_OS_WIN
+#ifdef Q_OS_WINCE
+#include "qserialport_wince_p.h"
+#elif defined (Q_OS_WIN)
 #include "qserialport_win_p.h"
 #elif defined (Q_OS_SYMBIAN)
 #include "qserialport_symbian_p.h"
@@ -59,6 +61,8 @@
 #  define SERIALPORT_BUFFERSIZE 16384
 #endif
 
+#include <QtCore/qdebug.h>
+
 QT_BEGIN_NAMESPACE
 
 QSerialPortPrivateData::QSerialPortPrivateData(QSerialPort *q)
@@ -66,13 +70,15 @@ QSerialPortPrivateData::QSerialPortPrivateData(QSerialPort *q)
     , readBuffer(SERIALPORT_BUFFERSIZE)
     , writeBuffer(SERIALPORT_BUFFERSIZE)
     , error(QSerialPort::NoError)
-    , inputBaudRate(0)
-    , outputBaudRate(0)
-    , dataBits(QSerialPort::UnknownDataBits)
-    , parity(QSerialPort::UnknownParity)
-    , stopBits(QSerialPort::UnknownStopBits)
-    , flow(QSerialPort::UnknownFlowControl)
+    , inputBaudRate(9600)
+    , outputBaudRate(9600)
+    , dataBits(QSerialPort::Data8)
+    , parity(QSerialPort::NoParity)
+    , stopBits(QSerialPort::OneStop)
+    , flowControl(QSerialPort::NoFlowControl)
     , policy(QSerialPort::IgnorePolicy)
+    , dataTerminalReady(false)
+    , requestToSend(false)
     , settingsRestoredOnClose(true)
     , q_ptr(q)
 {
@@ -114,10 +120,10 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
     port to the desired setting using the setBaudRate(), setDataBits(),
     setParity(), setStopBits(), and setFlowControl() methods.
 
-    The status of the control pinout signals is determined with the
-    isDataTerminalReady(), isRequestToSend, and pinoutSignals() methods. To
-    change the control line status, use the setDataTerminalReady(), and
-    setRequestToSend() methods.
+    There are a couple of properties to work with the pinout signals namely:
+    QSerialPort::dataTerminalReady, QSerialPort::requestToSend. It is also
+    possible to use the pinoutSignals() method to query the current pinout
+    signals set.
 
     Once you know that the ports are ready to read or write, you can
     use the read() or write() methods. Alternatively the
@@ -184,8 +190,9 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
     \enum QSerialPort::BaudRate
 
     This enum describes the baud rate which the communication device operates
-    with. Note: only the most common standard baud rates are listed in this
-    enum.
+    with.
+
+    \note Only the most common standard baud rates are listed in this enum.
 
     \value Baud1200     1200 baud.
     \value Baud2400     2400 baud.
@@ -195,7 +202,9 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
     \value Baud38400    38400 baud.
     \value Baud57600    57600 baud.
     \value Baud115200   115200 baud.
-    \value UnknownBaud  Unknown baud.
+    \value UnknownBaud  Unknown baud. This value is obsolete. It is provided to
+                        keep old source code working. We strongly advise against
+                        using it in new code.
 
     \sa QSerialPort::baudRate
 */
@@ -205,11 +214,21 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
 
     This enum describes the number of data bits used.
 
-    \value Data5            Five bits.
-    \value Data6            Six bits.
-    \value Data7            Seven bits
-    \value Data8            Eight bits.
-    \value UnknownDataBits  Unknown number of bits.
+    \value Data5            The number of data bits in each character is 5. It
+                            is used for Baudot code. It generally only makes
+                            sense with older equipment such as teleprinters.
+    \value Data6            The number of data bits in each character is 6. It
+                            is rarely used.
+    \value Data7            The number of data bits in each character is 7. It
+                            is used for true ASCII. It generally only makes
+                            sense with older equipment such as teleprinters.
+    \value Data8            The number of data bits in each character is 8. It
+                            is used for most kinds of data, as this size matches
+                            the size of a byte. It is almost universally used in
+                            newer applications.
+    \value UnknownDataBits  Unknown number of bits. This value is obsolete. It
+                            is provided to keep old source code working. We
+                            strongly advise against using it in new code.
 
     \sa QSerialPort::dataBits
 */
@@ -219,12 +238,23 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
 
     This enum describes the parity scheme used.
 
-    \value NoParity No parity.
-    \value EvenParity Even parity.
-    \value OddParity Odd parity.
-    \value SpaceParity Space parity.
-    \value MarkParity Mark parity.
-    \value UnknownParity Unknown parity.
+    \value NoParity         No parity bit it sent. This is the most common
+                            parity setting. Error detection is handled by the
+                            communication protocol.
+    \value EvenParity       The number of 1 bits in each character, including
+                            the parity bit, is always even.
+    \value OddParity        The number of 1 bits in each character, including
+                            the parity bit, is always odd. It ensures that at
+                            least one state transition occurs in each character.
+    \value SpaceParity      Space parity. The parity bit is sent in the space
+                            signal condition. It does not provide error
+                            detection information.
+    \value MarkParity       Mark parity. The parity bit is always set to the
+                            mark signal condition (logical 1). It does not
+                            provide error detection information.
+    \value UnknownParity    Unknown parity. This value is obsolete. It is
+                            provided to keep old source code working. We
+                            strongly advise against using it in new code.
 
     \sa QSerialPort::parity
 */
@@ -234,10 +264,12 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
 
     This enum describes the number of stop bits used.
 
-    \value OneStop 1 stop bit.
-    \value OneAndHalfStop 1.5 stop bits.
-    \value TwoStop 2 stop bits.
-    \value UnknownStopBits Unknown number of stop bit.
+    \value OneStop          1 stop bit.
+    \value OneAndHalfStop   1.5 stop bits. This is only for the Windows platform.
+    \value TwoStop          2 stop bits.
+    \value UnknownStopBits  Unknown number of stop bits. This value is obsolete.
+                            It is provided to keep old source code working. We
+                            strongly advise against using it in new code.
 
     \sa QSerialPort::stopBits
 */
@@ -247,10 +279,12 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
 
     This enum describes the flow control used.
 
-    \value NoFlowControl No flow control.
-    \value HardwareControl Hardware flow control (RTS/CTS).
-    \value SoftwareControl Software flow control (XON/XOFF).
-    \value UnknownFlowControl Unknown flow control.
+    \value NoFlowControl        No flow control.
+    \value HardwareControl      Hardware flow control (RTS/CTS).
+    \value SoftwareControl      Software flow control (XON/XOFF).
+    \value UnknownFlowControl   Unknown flow control. This value is obsolete. It
+                                is provided to keep old source code working. We
+                                strongly advise against using it in new code.
 
     \sa QSerialPort::flowControl
 */
@@ -261,8 +295,14 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
     This enum describes the possible RS-232 pinout signals.
 
     \value NoSignal                       No line active
-    \value TransmittedDataSignal          TxD (Transmitted Data).
-    \value ReceivedDataSignal             RxD (Received Data).
+    \value TransmittedDataSignal          TxD (Transmitted Data). This value is
+                                          obsolete. It is provided to keep old
+                                          source code working. We strongly
+                                          advise against using it in new code.
+    \value ReceivedDataSignal             RxD (Received Data). This value is
+                                          obsolete. It is provided to keep old
+                                          source code working. We strongly
+                                          advise against using it in new code.
     \value DataTerminalReadySignal        DTR (Data Terminal Ready).
     \value DataCarrierDetectSignal        DCD (Data Carrier Detect).
     \value DataSetReadySignal             DSR (Data Set Ready).
@@ -278,12 +318,13 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
 
 /*!
     \enum QSerialPort::DataErrorPolicy
+    \obsolete
 
     This enum describes the policies for the received symbols
     while parity errors were detected.
 
     \value SkipPolicy           Skips the bad character.
-    \value PassZeroPolicy       Replaces bad character to zero.
+    \value PassZeroPolicy       Replaces bad character with zero.
     \value IgnorePolicy         Ignores the error for a bad character.
     \value StopReceivingPolicy  Stops data reception on error.
     \value UnknownPolicy        Unknown policy.
@@ -298,25 +339,48 @@ int QSerialPortPrivateData::timeoutValue(int msecs, int elapsed)
     QSerialPort::error property.
 
     \value NoError              No error occurred.
+
     \value DeviceNotFoundError  An error occurred while attempting to
                                 open an non-existing device.
-    \value PermissionError      An error occurred while attempting to
-           open an already opened device by another process or a user not
-           having enough permission and credentials to open.
-    \value OpenError            An error occurred while attempting to
-           open an already opened device in this object.
-    \value ParityError Parity error detected by the hardware while reading data.
-    \value FramingError Framing error detected by the hardware while reading data.
-    \value BreakConditionError Break condition detected by the hardware on
-           the input line.
-    \value WriteError An I/O error occurred while writing the data.
-    \value ReadError An I/O error occurred while reading the data.
-    \value ResourceError An I/O error occurred when a resource becomes unavailable,
-           e.g. when the device is unexpectedly removed from the system.
-    \value UnsupportedOperationError The requested device operation is
-           not supported or prohibited by the running operating system.
-    \value UnknownError         An unidentified error occurred.
 
+    \value PermissionError      An error occurred while attempting to
+                                open an already opened device by another
+                                process or a user not having enough permission
+                                and credentials to open.
+
+    \value OpenError            An error occurred while attempting to open an
+                                already opened device in this object.
+
+    \value NotOpenError         This error occurs when an operation is executed
+                                that can only be successfully performed if the
+                                device is open. This value was introduced in
+                                QtSerialPort 5.2.
+
+    \value ParityError          Parity error detected by the hardware while
+                                reading data.
+
+    \value FramingError         Framing error detected by the hardware while
+                                reading data.
+
+    \value BreakConditionError  Break condition detected by the hardware on
+                                the input line.
+
+    \value WriteError           An I/O error occurred while writing the data.
+
+    \value ReadError            An I/O error occurred while reading the data.
+
+    \value ResourceError        An I/O error occurred when a resource becomes
+                                unavailable, e.g. when the device is
+                                unexpectedly removed from the system.
+
+    \value UnsupportedOperationError The requested device operation is not
+                                supported or prohibited by the running operating
+                                system.
+
+    \value TimeoutError         A timeout error occurred. This value was
+                                introduced in QtSerialPort 5.2.
+
+    \value UnknownError         An unidentified error occurred.
     \sa QSerialPort::error
 */
 
@@ -361,14 +425,15 @@ QSerialPort::QSerialPort(const QSerialPortInfo &serialPortInfo, QObject *parent)
 QSerialPort::~QSerialPort()
 {
     /**/
-    close();
+    if (isOpen())
+        close();
     delete d_ptr;
 }
 
 /*!
     Sets the \a name of the serial port.
 
-    The name of the serial port can be passed on as either a short name or
+    The name of the serial port can be passed as either a short name or
     the long system location if necessary.
 
     \sa portName(), QSerialPortInfo
@@ -391,9 +456,9 @@ void QSerialPort::setPort(const QSerialPortInfo &serialPortInfo)
 }
 
 /*!
-    Returns the name set by setPort() or to the QSerialPort constructors.
-    This name is short, i.e. it extract and convert out from the internal
-    variable system location of the device. Conversion algorithm is
+    Returns the name set by setPort() or passed to the QSerialPort constructor.
+    This name is short, i.e. it is extracted and converted from the internal
+    variable system location of the device. The conversion algorithm is
     platform specific:
     \table
     \header
@@ -405,7 +470,7 @@ void QSerialPort::setPort(const QSerialPortInfo &serialPortInfo)
            and returns the remainder of the string.
     \row
         \li Windows CE
-        \li Removes the postfix ":" from the system location
+        \li Removes the suffix ":" from the system location
            and returns the remainder of the string.
     \row
         \li Symbian
@@ -436,9 +501,13 @@ QString QSerialPort::portName() const
     \reimp
 
     Opens the serial port using OpenMode \a mode, and then returns true if
-    successful; otherwise returns false with and sets an error code which can be
+    successful; otherwise returns false and sets an error code which can be
     obtained by calling the error() method.
 
+    \note The method returns false if opening the port is successful, but could
+    not set any of the port settings successfully. In that case, the port is
+    closed automatically not to leave the port around with incorrect settings.
+
     \warning The \a mode has to be QIODevice::ReadOnly, QIODevice::WriteOnly,
     or QIODevice::ReadWrite. Other modes are unsupported.
 
@@ -461,26 +530,39 @@ bool QSerialPort::open(OpenMode mode)
     }
 
     clearError();
-    if (d->open(mode)) {
-        QIODevice::open(mode);
+    if (!d->open(mode))
+        return false;
 
-        d->dataTerminalReady = isDataTerminalReady();
-        d->requestToSend = isRequestToSend();
+    QIODevice::open(mode);
 
-        return true;
+    if (!d->setBaudRate()
+        || !d->setDataBits(d->dataBits)
+        || !d->setParity(d->parity)
+        || !d->setStopBits(d->stopBits)
+        || !d->setFlowControl(d->flowControl)) {
+        close();
+        return false;
     }
-    return false;
+
+    d->dataTerminalReady = isDataTerminalReady();
+    d->requestToSend = isRequestToSend();
+
+    return true;
 }
 
 /*!
     \reimp
 
+    \note The serial port has to be open before trying to close it; otherwise
+    sets the NotOpenError error code.
+
     \sa QIODevice::close()
 */
 void QSerialPort::close()
 {
     Q_D(QSerialPort);
     if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
         return;
     }
 
@@ -490,11 +572,12 @@ void QSerialPort::close()
 
 /*!
     \property QSerialPort::settingsRestoredOnClose
-    \brief the flag which allows to restore the previous settings while closing
+    \brief the flag which specifies to restore the previous settings when closing
     the serial port.
+    \obsolete
 
     If this flag is true, the settings will be restored; otherwise not.
-    The default state of the QSerialPort class is configured to restore the
+    The default state of the QSerialPort class is to restore the
     settings.
 */
 void QSerialPort::setSettingsRestoredOnClose(bool restore)
@@ -516,9 +599,9 @@ bool QSerialPort::settingsRestoredOnClose() const
 /*!
     \fn void QSerialPort::settingsRestoredOnCloseChanged(bool restore)
 
-    This signal is emitted after the flag which allows to restore the
+    This signal is emitted after the flag which specifies to restore the
     previous settings while closing the serial port has been changed. The new
-    flag which allows to restore the previous settings while closing the serial
+    flag which specifies to restore the previous settings while closing the serial
     port is passed as \a restore.
 
     \sa QSerialPort::settingsRestoredOnClose
@@ -528,38 +611,45 @@ bool QSerialPort::settingsRestoredOnClose() const
     \property QSerialPort::baudRate
     \brief the data baud rate for the desired direction
 
-    If the setting is successful, returns true; otherwise returns false and sets
-    an error code which can be obtained by accessing the value of the
-    QSerialPort::error property. To set the baud rate, use the enumeration
-    QSerialPort::BaudRate or any positive qint32 value.
+    If the setting is successful or set before opening the port, returns true;
+    otherwise returns false and sets an error code which can be obtained by
+    accessing the value of the QSerialPort::error property. To set the baud
+    rate, use the enumeration QSerialPort::BaudRate or any positive qint32
+    value.
 
-    \warning Only the AllDirections flag is support for setting this property on
-    Windows, Windows CE, and Symbian.
+    \note If the setting is set before opening the port, the actual serial port
+    setting is done automatically in the \l{QSerialPort::open()} method right
+    after that the opening of the port succeeds.
+
+    \warning Setting the AllDirections flag is only supported on
+    the Windows, Windows CE, and Symbian platforms.
 
     \warning Returns equal baud rate in any direction on Windows, Windows CE, and
     Symbian.
+
+    The default value is Baud9600, i.e. 9600 bits per second.
 */
-bool QSerialPort::setBaudRate(qint32 baudRate, Directions dir)
+bool QSerialPort::setBaudRate(qint32 baudRate, Directions directions)
 {
     Q_D(QSerialPort);
 
-    if (d->setBaudRate(baudRate, dir)) {
-        if (dir & QSerialPort::Input) {
+    if (!isOpen() || d->setBaudRate(baudRate, directions)) {
+        if (directions & QSerialPort::Input) {
             if (d->inputBaudRate != baudRate)
                 d->inputBaudRate = baudRate;
             else
-                dir &= ~QSerialPort::Input;
+                directions &= ~QSerialPort::Input;
         }
 
-        if (dir & QSerialPort::Output) {
+        if (directions & QSerialPort::Output) {
             if (d->outputBaudRate != baudRate)
                 d->outputBaudRate = baudRate;
             else
-                dir &= ~QSerialPort::Output;
+                directions &= ~QSerialPort::Output;
         }
 
-        if (dir)
-            emit baudRateChanged(baudRate, dir);
+        if (directions)
+            emit baudRateChanged(baudRate, directions);
 
         return true;
     }
@@ -567,20 +657,20 @@ bool QSerialPort::setBaudRate(qint32 baudRate, Directions dir)
     return false;
 }
 
-qint32 QSerialPort::baudRate(Directions dir) const
+qint32 QSerialPort::baudRate(Directions directions) const
 {
     Q_D(const QSerialPort);
-    if (dir == QSerialPort::AllDirections)
+    if (directions == QSerialPort::AllDirections)
         return d->inputBaudRate == d->outputBaudRate ?
-                    d->inputBaudRate : QSerialPort::UnknownBaud;
-    return dir & QSerialPort::Input ? d->inputBaudRate : d->outputBaudRate;
+                    d->inputBaudRate : -1;
+    return directions & QSerialPort::Input ? d->inputBaudRate : d->outputBaudRate;
 }
 
 /*!
-    \fn void QSerialPort::baudRateChanged(qint32 baudRate, Directions dir)
+    \fn void QSerialPort::baudRateChanged(qint32 baudRate, Directions directions)
 
     This signal is emitted after the baud rate has been changed. The new baud
-    rate is passed as \a baudRate and directions as \a dir.
+    rate is passed as \a baudRate and directions as \a directions.
 
     \sa QSerialPort::baudRate
 */
@@ -589,15 +679,21 @@ qint32 QSerialPort::baudRate(Directions dir) const
     \property QSerialPort::dataBits
     \brief the data bits in a frame
 
-    If the setting is successful, returns true; otherwise returns false and sets
-    an error code which can be obtained by accessing the value of the
-    QSerialPort::error property.
+    If the setting is successful or set before opening the port, returns
+    true; otherwise returns false and sets an error code which can be obtained
+    by accessing the value of the QSerialPort::error property.
+
+    \note If the setting is set before opening the port, the actual serial port
+    setting is done automatically in the \l{QSerialPort::open()} method right
+    after that the opening of the port succeeds.
+
+    The default value is Data8, i.e. 8 data bits.
 */
 bool QSerialPort::setDataBits(DataBits dataBits)
 {
     Q_D(QSerialPort);
 
-    if (d->setDataBits(dataBits)) {
+    if (!isOpen() || d->setDataBits(dataBits)) {
         if (d->dataBits != dataBits) {
             d->dataBits = dataBits;
             emit dataBitsChanged(d->dataBits);
@@ -628,15 +724,21 @@ QSerialPort::DataBits QSerialPort::dataBits() const
     \property QSerialPort::parity
     \brief the parity checking mode
 
-    If the setting is successful, returns true; otherwise returns false and sets
-    an error code which can be obtained by accessing the value of the
-    QSerialPort::error property.
+    If the setting is successful or set before opening the port, returns true;
+    otherwise returns false and sets an error code which can be obtained by
+    accessing the value of the QSerialPort::error property.
+
+    \note If the setting is set before opening the port, the actual serial port
+    setting is done automatically in the \l{QSerialPort::open()} method right
+    after that the opening of the port succeeds.
+
+    The default value is NoParity, i.e. no parity.
 */
 bool QSerialPort::setParity(Parity parity)
 {
     Q_D(QSerialPort);
 
-    if (d->setParity(parity)) {
+    if (!isOpen() || d->setParity(parity)) {
         if (d->parity != parity) {
             d->parity = parity;
             emit parityChanged(d->parity);
@@ -666,15 +768,21 @@ QSerialPort::Parity QSerialPort::parity() const
     \property QSerialPort::stopBits
     \brief the number of stop bits in a frame
 
-    If the setting is successful, returns true; otherwise returns false and
-    sets an error code which can be obtained by accessing the value of the
-    QSerialPort::error property.
+    If the setting is successful or set before opening the port, returns true;
+    otherwise returns false and sets an error code which can be obtained by
+    accessing the value of the QSerialPort::error property.
+
+    \note If the setting is set before opening the port, the actual serial port
+    setting is done automatically in the \l{QSerialPort::open()} method right
+    after that the opening of the port succeeds.
+
+    The default value is OneStop, i.e. 1 stop bit.
 */
 bool QSerialPort::setStopBits(StopBits stopBits)
 {
     Q_D(QSerialPort);
 
-    if (d->setStopBits(stopBits)) {
+    if (!isOpen() || d->setStopBits(stopBits)) {
         if (d->stopBits != stopBits) {
             d->stopBits = stopBits;
             emit stopBitsChanged(d->stopBits);
@@ -704,18 +812,24 @@ QSerialPort::StopBits QSerialPort::stopBits() const
     \property QSerialPort::flowControl
     \brief the desired flow control mode
 
-    If the setting is successful, returns true; otherwise returns false and sets
-    an error code which can be obtained by accessing the value of the
-    QSerialPort::error property.
+    If the setting is successful or set before opening the port, returns true;
+    otherwise returns false and sets an error code which can be obtained by
+    accessing the value of the QSerialPort::error property.
+
+    \note If the setting is set before opening the port, the actual serial port
+    setting is done automatically in the \l{QSerialPort::open()} method right
+    after that the opening of the port succeeds.
+
+    The default value is NoFlowControl, i.e. no flow control.
 */
-bool QSerialPort::setFlowControl(FlowControl flow)
+bool QSerialPort::setFlowControl(FlowControl flowControl)
 {
     Q_D(QSerialPort);
 
-    if (d->setFlowControl(flow)) {
-        if (d->flow != flow) {
-            d->flow = flow;
-            emit flowControlChanged(d->flow);
+    if (!isOpen() || d->setFlowControl(flowControl)) {
+        if (d->flowControl != flowControl) {
+            d->flowControl = flowControl;
+            emit flowControlChanged(d->flowControl);
         }
         return true;
     }
@@ -726,7 +840,7 @@ bool QSerialPort::setFlowControl(FlowControl flow)
 QSerialPort::FlowControl QSerialPort::flowControl() const
 {
     Q_D(const QSerialPort);
-    return d->flow;
+    return d->flowControl;
 }
 
 /*!
@@ -742,16 +856,26 @@ QSerialPort::FlowControl QSerialPort::flowControl() const
     \property QSerialPort::dataTerminalReady
     \brief the state (high or low) of the line signal DTR
 
-    If the setting is successful, returns true; otherwise returns false.
+    Returns true on success, false otherwise.
     If the flag is true then the DTR signal is set to high; otherwise low.
 
+    \note The serial port has to be open before trying to set or get this
+    property; otherwise false is returned and the error code is set to
+    NotOpenError.
+
     \sa pinoutSignals()
 */
 bool QSerialPort::setDataTerminalReady(bool set)
 {
     Q_D(QSerialPort);
 
-    bool retval = d->setDataTerminalReady(set);
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
+    const bool retval = d->setDataTerminalReady(set);
     if (retval && (d->dataTerminalReady != set)) {
         d->dataTerminalReady = set;
         emit dataTerminalReadyChanged(set);
@@ -762,7 +886,7 @@ bool QSerialPort::setDataTerminalReady(bool set)
 
 bool QSerialPort::isDataTerminalReady()
 {
-    Q_D(const QSerialPort);
+    Q_D(QSerialPort);
     return d->pinoutSignals() & QSerialPort::DataTerminalReadySignal;
 }
 
@@ -780,16 +904,26 @@ bool QSerialPort::isDataTerminalReady()
     \property QSerialPort::requestToSend
     \brief the state (high or low) of the line signal RTS
 
-    If the setting is successful, returns true; otherwise returns false.
+    Returns true on success, false otherwise.
     If the flag is true then the RTS signal is set to high; otherwise low.
 
+    \note The serial port has to be open before trying to set or get this
+    property; otherwise false is returned and the error code is set to
+    NotOpenError.
+
     \sa pinoutSignals()
 */
 bool QSerialPort::setRequestToSend(bool set)
 {
     Q_D(QSerialPort);
 
-    bool retval = d->setRequestToSend(set);
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
+    const bool retval = d->setRequestToSend(set);
     if (retval && (d->requestToSend != set)) {
         d->requestToSend = set;
         emit requestToSendChanged(set);
@@ -800,7 +934,7 @@ bool QSerialPort::setRequestToSend(bool set)
 
 bool QSerialPort::isRequestToSend()
 {
-    Q_D(const QSerialPort);
+    Q_D(QSerialPort);
     return d->pinoutSignals() & QSerialPort::RequestToSendSignal;
 }
 
@@ -821,16 +955,25 @@ bool QSerialPort::isRequestToSend()
     desired signal by applying a mask "AND", where the mask is
     the desired enumeration value from QSerialPort::PinoutSignals.
 
-    Note that, this method performs a system call, thus ensuring that the line
-    signal states are returned properly. This is necessary when the underlying
+    \note This method performs a system call, thus ensuring that the line signal
+    states are returned properly. This is necessary when the underlying
     operating systems cannot provide proper notifications about the changes.
 
-    \sa isDataTerminalReady(), isRequestToSend, setDataTerminalReady(),
-    setRequestToSend()
+    \note The serial port has to be open before trying to get the pinout
+    signals; otherwise returns NoSignal and sets the NotOpenError error code.
+
+    \sa QSerialPort::dataTerminalReady, QSerialPort::requestToSend
 */
 QSerialPort::PinoutSignals QSerialPort::pinoutSignals()
 {
-    Q_D(const QSerialPort);
+    Q_D(QSerialPort);
+
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return QSerialPort::NoSignal;
+    }
+
     return d->pinoutSignals();
 }
 
@@ -846,28 +989,48 @@ QSerialPort::PinoutSignals QSerialPort::pinoutSignals()
     returned to the event loop. In the absence of an event loop, call
     waitForBytesWritten() instead.
 
+    \note The serial port has to be open before trying to flush any buffered
+    data; otherwise returns false and sets the NotOpenError error code.
+
     \sa write(), waitForBytesWritten()
 */
 bool QSerialPort::flush()
 {
     Q_D(QSerialPort);
+
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
     return d->flush();
 }
 
 /*!
     Discards all characters from the output or input buffer, depending on
-    a given direction \a dir. Including clear an internal class buffers and
+    given directions \a directions. This includes clearing the internal class buffers and
     the UART (driver) buffers. Also terminate pending read or write operations.
     If successful, returns true; otherwise returns false.
+
+    \note The serial port has to be open before trying to clear any buffered
+    data; otherwise returns false and sets the NotOpenError error code.
 */
-bool QSerialPort::clear(Directions dir)
+bool QSerialPort::clear(Directions directions)
 {
     Q_D(QSerialPort);
-    if (dir & Input)
+
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
+    if (directions & Input)
         d->readBuffer.clear();
-    if (dir & Output)
+    if (directions & Output)
         d->writeBuffer.clear();
-    return d->clear(dir);
+    return d->clear(directions);
 }
 
 /*!
@@ -895,21 +1058,35 @@ bool QSerialPort::clear(Directions dir)
 bool QSerialPort::atEnd() const
 {
     Q_D(const QSerialPort);
-    return QIODevice::atEnd() && (!isOpen() || (d->bytesAvailable() == 0));
+    return QIODevice::atEnd() && (!isOpen() || (d->readBuffer.size() == 0));
 }
 
 /*!
     \property QSerialPort::dataErrorPolicy
-    \brief the error policy how the process receives the character in case of
-    parity error detection.
+    \brief the error policy for how the process receives characters in the case where
+    a parity error is detected.
+    \obsolete
 
     If the setting is successful, returns true; otherwise returns false. The
     default policy set is IgnorePolicy.
+
+    \note The serial port has to be open before trying to set this property;
+    otherwise returns false and sets the NotOpenError error code. This is a bit
+    unusual as opposed to the regular Qt property settings of a class. However,
+    this is a special use case since the property is set through the interaction
+    with the kernel and hardware. Hence, the two scenarios cannot be completely
+    compared to each other.
 */
 bool QSerialPort::setDataErrorPolicy(DataErrorPolicy policy)
 {
     Q_D(QSerialPort);
 
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
     const bool ret = d->policy == policy || d->setDataErrorPolicy(policy);
     if (ret && (d->policy != policy)) {
         d->policy = policy;
@@ -927,10 +1104,11 @@ QSerialPort::DataErrorPolicy QSerialPort::dataErrorPolicy() const
 
 /*!
     \fn void QSerialPort::dataErrorPolicyChanged(DataErrorPolicy policy)
+    \obsolete
 
-    This signal is emitted after the error policy how the process receives the
-    character in case of parity error detection has been changed. The new error
-    policy how the process receives the character in case of parity error
+    This signal is emitted after the error policy for how the process receives
+    characters in case of parity error detection has been changed. The new error
+    policy for how the process receives the character in case of parity error
     detection is passed as \a policy.
 
     \sa QSerialPort::dataErrorPolicy
@@ -961,8 +1139,8 @@ void QSerialPort::clearError()
 /*!
     \fn void QSerialPort::error(SerialPortError error)
 
-    This signal is emitted after the error has been changed. The new erroris
-    passed as \a error.
+    This signal is emitted after the error has been changed. The new error
+    is passed as \a error.
 
     \sa QSerialPort::error
 */
@@ -988,14 +1166,14 @@ qint64 QSerialPort::readBufferSize() const
     size bytes.
 
     If the buffer size is limited to a certain size, QSerialPort
-    will not buffer more than this size of data. Exceptionally, a buffer
+    will not buffer more than this size of data. The special case of a buffer
     size of 0 means that the read buffer is unlimited and all
     incoming data is buffered. This is the default.
 
     This option is useful if the data is only read at certain points
     in time (for instance in a real-time streaming application) or if the serial
     port should be protected against receiving too much data, which may
-    eventually causes that the application runs out of memory.
+    eventually cause the application to run out of memory.
 
     \sa readBufferSize(), read()
 */
@@ -1028,7 +1206,7 @@ bool QSerialPort::isSequential() const
 qint64 QSerialPort::bytesAvailable() const
 {
     Q_D(const QSerialPort);
-    return d->bytesAvailable() + QIODevice::bytesAvailable();
+    return d->readBuffer.size() + QIODevice::bytesAvailable();
 }
 
 /*!
@@ -1057,7 +1235,7 @@ qint64 QSerialPort::bytesToWrite() const
 bool QSerialPort::canReadLine() const
 {
     Q_D(const QSerialPort);
-    const bool hasLine = (d->bytesAvailable() > 0) && d->readBuffer.canReadLine();
+    const bool hasLine = (d->readBuffer.size() > 0) && d->readBuffer.canReadLine();
     return hasLine || QIODevice::canReadLine();
 }
 
@@ -1080,6 +1258,18 @@ bool QSerialPort::waitForReadyRead(int msecs)
     return d->waitForReadyRead(msecs);
 }
 
+/*!
+    \fn Handle QSerialPort::handle() const
+    \since 5.2
+
+    If the platform is supported and the serial port is open, returns the native
+    serial port handle; otherwise returns -1.
+
+    \warning This function is for expert use only; use it at your own risk.
+    Furthermore, this function carries no compatibility promise between minor
+    Qt releases.
+*/
+
 /*!
     \reimp
 */
@@ -1100,11 +1290,21 @@ bool QSerialPort::waitForBytesWritten(int msecs)
     If the duration is non zero then zero bits are transmitted within a certain
     period of time depending on the implementation.
 
+    \note The serial port has to be open before trying to send a break
+    duration; otherwise returns false and sets the NotOpenError error code.
+
     \sa setBreakEnabled()
 */
 bool QSerialPort::sendBreak(int duration)
 {
     Q_D(QSerialPort);
+
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
     return d->sendBreak(duration);
 }
 
@@ -1114,11 +1314,21 @@ bool QSerialPort::sendBreak(int duration)
 
     If \a set is true then enables the break transmission; otherwise disables.
 
+    \note The serial port has to be open before trying to set break enabled;
+    otherwise returns false and sets the NotOpenError error code.
+
     \sa sendBreak()
 */
 bool QSerialPort::setBreakEnabled(bool set)
 {
     Q_D(QSerialPort);
+
+    if (!isOpen()) {
+        setError(QSerialPort::NotOpenError);
+        qWarning("%s: device not open", Q_FUNC_INFO);
+        return false;
+    }
+
     return d->setBreakEnabled(set);
 }
 
@@ -1128,7 +1338,7 @@ bool QSerialPort::setBreakEnabled(bool set)
 qint64 QSerialPort::readData(char *data, qint64 maxSize)
 {
     Q_D(QSerialPort);
-    return d->readFromBuffer(data, maxSize);
+    return d->readBuffer.read(data, maxSize);
 }
 
 /*!
@@ -1145,7 +1355,11 @@ qint64 QSerialPort::readLineData(char *data, qint64 maxSize)
 qint64 QSerialPort::writeData(const char *data, qint64 maxSize)
 {
     Q_D(QSerialPort);
-    return d->writeToBuffer(data, maxSize);
+
+    ::memcpy(d->writeBuffer.reserve(maxSize), data, maxSize);
+    if (!d->writeBuffer.isEmpty())
+        d->startWriting();
+    return maxSize;
 }
 
 void QSerialPort::setError(QSerialPort::SerialPortError serialPortError, const QString &errorString)
